__author__ = 'tomas'

import sys

from PyQt4 import QtGui
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas

import Data

# from simple_viewer import Ui_MainWindow
from simple_viewer import Ui_Form

# class Viewer_3D(QtGui.QMainWindow):
class Viewer_3D(QtGui.QWidget):
    """Main class of the programm."""

    def __init__(self, data, range=False, parent=None):
        QtGui.QWidget.__init__(self, parent)
        # self.ui = Ui_MainWindow()
        self.ui = Ui_Form()
        self.ui.setupUi(self)

        self.actual_slice = 0
        self.data = data
        self.n_slices = self.data.shape[0]
        self.range = range

        # seting up the figure
        self.figure = plt.figure()
        self.axes = self.figure.add_axes([0, 0, 1, 1])
        self.canvas = FigureCanvas(self.figure)

        # layout = QtGui.QVBoxLayout()
        # layout.addWidget(self.canvas, 1)
        # self.setLayout(layout)
        # conenction to wheel events
        self.canvas.mpl_connect('scroll_event', self.on_scroll)

        self.update_figures()

        # self.data = data
        # adding widget for displaying image data
        # self.form_widget = Form_widget.Form_widget(self)
        data_viewer_layout = QtGui.QHBoxLayout()
        # data_viewer_layout.addWidget(self.form_widget)
        data_viewer_layout.addWidget(self.canvas)
        self.ui.viewer_F.setLayout(data_viewer_layout)

        # seting up the range of the scrollbar to cope with the number of slices
        self.ui.slice_scrollB.setMaximum(self.n_slices - 1)

        # connecting slider
        self.ui.slice_scrollB.valueChanged.connect(self.slider_changed)


    def update_figures(self):
        if self.range:
            vmin = self.data.min()
            vmax = self.data.max()
        else:
            vmin = 0
            vmax = 255
        slice = self.data[self.actual_slice, :, :]

        plt.figure(self.figure.number)
        plt.subplot(111)
        # self.figure.gca().cla()  # clearing the contours, just to be sure
        plt.imshow(slice, 'gray', interpolation='nearest', vmin=vmin, vmax=vmax)

        self.canvas.draw()


    def slider_changed(self, val):
        self.slice_change(val)
        self.actual_slice = val
        self.update_figures()


    def slice_change(self, val):
        self.ui.slice_scrollB.setValue(val)
        self.ui.slice_number_LBL.setText('slice # = %i' % (val + 1))


    def next_slice(self):
        self.actual_slice += 1
        if self.actual_slice >= self.n_slices:
            self.actual_slice = 0


    def prev_slice(self):
        self.actual_slice -= 1
        if self.actual_slice < 0:
            self.actual_slice = self.n_slices - 1


    def on_scroll(self, event):
        '''mouse wheel is used for setting slider value'''
        if event.button == 'up':
            self.next_slice()
        if event.button == 'down':
            self.prev_slice()
        self.update_figures()
        self.slice_change(self.actual_slice)


# def run(data):
#     app2 = QtGui.QApplication(sys.argv)
#     view = Viewer_3D(data)
#     view.show()
#     sys.exit(app2.exec_())


################################################################################
################################################################################
if __name__ == '__main__':

    # preparing data -----------------------------
    # size = 100
    # n_slices = 4
    # data = np.zeros((n_slices, size, size))
    # step = size / n_slices
    # for i in range(n_slices):
    #     data[i, i * step:(i + 1) * step, :] = 150

    # fname = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_183_46324212_venous_5.0_B30f-.pklz'
    fname = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_183_46324212_arterial_5.0_B30f-.pklz'

    data = Data.Data()
    data.load_data(fname)

    # data.data = np.load('/home/tomas/Data/labels.npy')
    data.data = np.load('labels_filt.npy')

    app = QtGui.QApplication(sys.argv)
    viewer = Viewer_3D(data.data, range=True)
    viewer.show()
    sys.exit(app.exec_())


__author__ = 'root'

import numpy as np

import logging
logger = logging.getLogger(__name__)
logging.basicConfig()

import pickle

import tools

# definition of different views
VIEW_TABLE = {'axial': (2,1,0),
              'sagittal': (1,0,2),
              'coronal': (2,0,1)}

class Data(object):

    def __init__(self, data=None, mask=None, filename=None):
        self.mask = mask
        self.voxel_size = None
        self.slab = None
        self.shape = None
        self.orig_shape = None
        self.objects = None  # array where each object has unique label
        # self.object_centroids = None
        self.labels_v = None
        self.n_rows = None
        self.n_cols = None
        self.n_slices = None
        self.models = None  # color models

        # properties
        self.__data = data
        self.__labels = None  # array of labeled results (only 3 classes: healthy, hypodense and hyperdense
        self.__labels_aview = None
        self.__labels_filt = None  # array of labeled data that are filtered, e.g. filtered by area, compactness etc.
        self.__labels_filt_aview = None
        self.__object_centers = None  # array of object centers
        self.__object_centers_filt = None
        self.__object_centers_list = list()  # list of object centers
        self.__lesions = list()  # list of lesions, set it with Lesions.extract_lesions(self.labels)
        self.__lesions_filt = list()

        self.filename = filename
        self.loaded = False

        self.win_level = 50
        self.win_width = 300

        # seting up views
        self.actual_view = 'axial'
        self.act_transposition = VIEW_TABLE[self.actual_view]
        if self.data is not None:
            self.data_aview = self.data.transpose(self.act_transposition)
        else:
            self.data_aview = None

    @property
    def data(self):
        return self.__data

    @data.setter
    def data(self, x):
        self.__data = x
        self.labels = np.zeros(x.shape)
        self.objects = np.zeros(x.shape)
        self.object_centers = np.zeros(x.shape)
        self.orig_shape = x.shape
        self.shape = x.shape

    @property
    def labels(self):
        return self.__labels

    @labels.setter
    def labels(self, x):
        self.__labels = x
        self.__labels_aview = self.__labels.transpose(self.act_transposition)
        self.__labels_filt = x
        self.__labels_filt_aview = self.__labels_filt.transpose(self.act_transposition)

    @labels.deleter
    def labels(self):
        del self.__labels
        del self.__labels_aview

    @property
    def labels_aview(self):
        return self.__labels_aview

    @labels_aview.setter
    def labels_aview(self, x):
        self.__labels_aview = x

    @property
    def labels_filt(self):
        return self.__labels_filt

    @labels_filt.setter
    def labels_filt(self, x):
        self.__labels_filt = x
        self.__labels_filt_aview = x.transpose(self.act_transposition)

        # print 'setting lesion_filt ...',
        # self.__lesions_filt = [x for x in self.lesions if x.label in np.unique(self.__labels_filt)]
        # print 'done'

        # print 'setting object_centers_filt...',
        # self.__object_centers_filt = np.zeros(self.objects.shape, dtype=np.bool)
        # for i in self.__lesions_filt:
        #     try:
        #         idx = (np.round(i.center)).astype(np.int)
        #         self.__object_centers_filt[idx[0], idx[1], idx[2]] = 1
        #     except:
        #         pass
        # print 'done'

    @labels_filt.deleter
    def labels_filt(self):
        del self.__labels_filt

    @property
    def labels_filt_aview(self):
        return self.__labels_filt_aview

    @labels_filt_aview.setter
    def labels_filt_aview(self, x):
        self.__labels_filt_aview = x

    # @property
    # def user_seeds(self):
    #     return self.__user_seeds

    @property
    def lesions(self):
        return self.__lesions

    @lesions.setter
    def lesions(self, x):
        self.__lesions = x
        self.__lesions_filt = x[:]
        self.__object_centers_list = [l.center for l in self.lesions]
        self.__object_centers = np.zeros(self.objects.shape, dtype=np.bool)
        for i in x:
            try:
                idx = (np.round(i.center)).astype(np.int)
                self.__object_centers[idx[0], idx[1], idx[2]] = 1
            except:
                pass

    @property
    def lesions_filt(self):
        return self.__lesions_filt

    @lesions_filt.setter
    def lesions_filt(self, x):
        self.__lesions_filt = x

        centers = [l.center for l in self.lesions_filt]

        centers_filt = np.zeros(self.objects.shape, dtype=np.bool)
        for i in centers:
            try:
                idx = np.round(i).astype(np.int)
                centers_filt[idx[0], idx[1], idx[2]] = 1
            except:
                pass
        self.__object_centers_filt = centers_filt

    @property
    def object_centers(self):
        return self.__object_centers

    @object_centers.setter
    def object_centers(self, x):
        self.__object_centers = x

    @property
    def object_centers_filt(self):
        return self.__object_centers_filt

    @object_centers_filt.setter
    def object_centers_filt(self, x):
        self.__object_centers_filt = x

    @property
    def object_centers_list(self):
        return self.__object_centers_list

    @object_centers_list.setter
    def object_centers_list(self, x):
        self.__object_centers_list = x

    def append_lesion(self, les):
        self.__lesions.append(les)
        self.__object_centers[les.center[0], les.center[1], les.center[2]] = 1
        self.__object_centers_list.append(les.center)

    # def load_data(self, filename, slice_idx=-1):
    #     self.filename = filename
    #     fcontent = None
    #     try:
    #         import gzip
    #         f = gzip.open(self.filename, 'rb')
    #         fcontent = f.read()
    #         f.close()
    #     except Exception as e:
    #         logger.warning("Input gzip exception: " + str(e))
    #         f = open(self.filename, 'rb')
    #         fcontent = f.read()
    #         f.close()
    #     data_dict = pickle.loads(fcontent)
    #
    #     self.data = data_dict['data3d']
    #     self.mask = data_dict['segmentation']
    #
    #     self.voxel_size = data_dict['voxelsize_mm']
    #
    #     if slice_idx != -1:
    #         self.data = self.data[:, :, slice_idx]
    #         self.mask = self.mask[:, :, slice_idx]
    #
    #     # data_s = 200 * np.triu(np.ones((100,120), dtype=np.int))
    #     # data = np.dstack((data_s, data_s))
    #     # self.data = np.rollaxis(data, 2, 0)
    #     # self.mask = np.ones_like(self.data)
    #
    #     self.data_vis = self.data
    #     self.data_aview = self.data.transpose(self.act_transposition)
    #
    #     self.orig_shape = self.data.shape
    #     self.n_slices, self.n_rows, self.n_cols = self.orig_shape
    #     self.shape = self.data.shape
    #
    #     self.loaded = True

    def create_data(self, datap, filename):
        if datap is None and filename is None:
            raise AttributeError('No data nor filename given.')

        # if filename is not None:
        #     self.filename = filename
        #     fcontent = None
        #     try:
        #         import gzip
        #         f = gzip.open(self.filename, 'rb')
        #         fcontent = f.read()
        #         f.close()
        #     except Exception as e:
        #         logger.warning("Input gzip exception: " + str(e))
        #         f = open(self.filename, 'rb')
        #         fcontent = f.read()
        #         f.close()
        #     datap = pickle.loads(fcontent)
        # else:
        self.filename = filename

        self.data = datap['data3d']
        self.mask = datap['segmentation']
        self.voxel_size = datap['voxelsize_mm']
        self.slab = datap['slab']

        self.data_vis = self.data
        self.data_aview = self.data.transpose(self.act_transposition)

        self.orig_shape = self.data.shape
        self.n_slices, self.n_rows, self.n_cols = self.orig_shape
        self.shape = self.data.shape

        self.loaded = True

if __name__ == '__main__':
    d = Data()
    d.labels = np.array([[[1, 1, 1]]])
    print d.labels

    d.labels = np.array([[[2, 2, 2]]])
    print d.labels


from __future__ import division

__author__ = 'tomas'

import sys
from PyQt4 import QtGui, QtCore

import numpy as np
import matplotlib.pyplot as plt
import scipy.ndimage.morphology as scindimor
import scipy.ndimage.measurements as scindimea
import scipy.ndimage.interpolation as scindiint
import scipy.ndimage as scindi
import scipy.stats as scista

import skimage.segmentation as skiseg
import skimage.morphology as skimor
import skimage.filter as skifil
import skimage.exposure as skiexp
import skimage.measure as skimea
import skimage.transform as skitra

# import cv2
# import pygco

import tools
import py3DSeedEditor
# from mayavi import mlab

import TumorVisualiser
import Viewer_3D

from sklearn import metrics
from sklearn.cluster import KMeans

import pickle

import io3d

import ConfigParser

import logging
logger = logging.getLogger(__name__)
logging.basicConfig()


from lession_editor_GUI_slim import Ui_MainWindow
import Form_widget
from hist_widget import Hist_widget
from objects_widget import Objects_widget
import My_table_model as mtm
import area_hist_widget as ahw
# import Computational_core
import computational_core as coco
import data_view_widget
import Lesion
import Data

# constants definition
SHOW_IM = 0
SHOW_LABELS = 1
SHOW_CONTOURS = 2
# SHOW_FILTERED_LABELS = 3

MODE_VIEWING = 0
MODE_ADDING = 1

class LessionEditor(QtGui.QMainWindow):
    """Main class of the programm."""


    # def __init__(self, datap_1=None, datap_2=None, fname_1=None, fname_2=None, disp_smoothed=False, parent=None):
    def __init__(self, datap1=None, datap2=None):

        QtGui.QWidget.__init__(self, parent=None)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        # uprava stylu pro lepsi vizualizaci splitteru
        QtGui.QApplication.setStyle(QtGui.QStyleFactory.create('Cleanlooks'))

        # self.im = im
        # self.labels = labels
        self.mode = MODE_VIEWING
        self.show_view_L = True
        self.show_view_R = False

        self.view_L = data_view_widget.SliceBox(self)
        self.view_R = data_view_widget.SliceBox(self)

        self.data_1 = Data.Data()
        self.data_2 = Data.Data()

        # self.healthy_label = healthy_label
        # self.hypo_label = hypo_label
        # self.hyper_label = hyper_label
        # self.disp_smoothed = disp_smoothed
        # self.view_L_curr_idx = 0
        # self.view_R_curr_idx = 0

        self.show_mode_L = SHOW_IM
        self.show_mode_R = SHOW_IM

        # load parameters
        self.params = {
            'win_l': 50,
            'win_w': 350,
            'alpha': 4,
            'beta': 1,
            'zoom': 0,
            'scale': 0.25,
            'perc': 30,
            'k_std_h': 3,
            'healthy_simple_estim': 0,
            'prob_w': 0.0001,
            'unaries_as_cdf': 0,
            'bgd_label': 0,
            'hypo_label': 1,
            'healthy_label': 2,
            'hyper_label': 3
            # 'voxel_size': (1, 1, 1)
        }
        # self.params.update(self.load_parameters())
        # self.win_l = self.params['win_level']
        # self.win_w = self.params['win_width']

        self.actual_slice_L = 0
        self.actual_slice_R = 0

        self.selected_objects_labels = None  # list of objects' labels selected in tableview

        # self.voxel_size = self.params['voxel_size']
        self.view_widget_width = 50
        self.two_views = False

        self.hist_widget = Hist_widget()
        self.hist_widget.heal_parameter_changed.connect(self.update_models_from_widget)
        self.hist_widget.hypo_parameter_changed.connect(self.update_models_from_widget)
        self.hist_widget.hyper_parameter_changed.connect(self.update_models_from_widget)

        self.setup_data(datap1, datap2)

       # creating object widget, linking its sliders and buttons
        self.objects_widget = Objects_widget()
        self.params['compactness_step'] = self.objects_widget.ui.min_compactness_SL.singleStep() / self.objects_widget.ui.min_compactness_SL.maximum()
        self.objects_widget.area_RS.endValueChanged.connect(self.object_slider_changed)
        self.objects_widget.area_RS.startValueChanged.connect(self.object_slider_changed)
        self.objects_widget.density_RS.endValueChanged.connect(self.object_slider_changed)
        self.objects_widget.density_RS.startValueChanged.connect(self.object_slider_changed)
        self.objects_widget.ui.min_compactness_SL.valueChanged.connect(self.min_compactness_SL_changed)
        # connecting min_compactness line edit and slider
        ww_val = QtGui.QIntValidator(0, 1)
        self.objects_widget.ui.min_compactness_LE.setValidator(ww_val)
        self.objects_widget.ui.min_compactness_LE.textChanged.connect(self.min_compactness_LE_changed)
        # button callbacks
        self.objects_widget.ui.add_obj_BTN.clicked.connect(self.add_obj_BTN_callback)
        self.objects_widget.ui.remove_obj_BTN.clicked.connect(self.remove_obj_BTN_callback)
        self.params['max_area'] = self.objects_widget.area_RS.end()
        self.params['min_area'] = self.objects_widget.area_RS.start()
        self.params['max_density'] = self.objects_widget.density_RS.end()
        self.params['min_density'] = self.objects_widget.density_RS.start()
        self.params['min_compactness'] = self.objects_widget.ui.min_compactness_SL.value() * self.params['compactness_step']

        self.ui.action_load_serie_1.triggered.connect(lambda: self.action_load_serie_callback(1))
        self.ui.action_load_serie_2.triggered.connect(lambda: self.action_load_serie_callback(2))

        self.ui.action_circle.triggered.connect(self.action_circle_callback)
        self.ui.action_show_color_model.triggered.connect(self.action_show_color_model_callback)
        self.ui.action_show_object_list.triggered.connect(self.action_show_object_list_callback)
        self.ui.action_run.triggered.connect(self.run_callback)

        # combo boxes - for figure views
        self.ui.figure_L_CB.currentIndexChanged.connect(self.figure_L_CB_callback)
        self.ui.figure_R_CB.currentIndexChanged.connect(self.figure_R_CB_callback)

        data_viewer_layout = QtGui.QHBoxLayout()
        # data_viewer_layout.addWidget(self.form_widget)
        data_viewer_layout.addWidget(self.view_L)
        data_viewer_layout.addWidget(self.view_R)
        self.ui.viewer_F.setLayout(data_viewer_layout)

        # connecting callbacks ----------------------------------
        self.ui.view_L_BTN.clicked.connect(self.view_L_callback)
        self.ui.view_R_BTN.clicked.connect(self.view_R_callback)

        # show image data
        self.ui.show_im_L_BTN.clicked.connect(self.show_im_L_callback)
        self.ui.show_im_R_BTN.clicked.connect(self.show_im_R_callback)

        # show label data
        self.ui.show_labels_L_BTN.clicked.connect(self.show_labels_L_callback)
        self.ui.show_labels_R_BTN.clicked.connect(self.show_labels_R_callback)

        # show contours data
        self.ui.show_contours_L_BTN.clicked.connect(self.show_contours_L_callback)
        self.ui.show_contours_R_BTN.clicked.connect(self.show_contours_R_callback)

        # connecting scrollbars
        self.ui.slice_C_SB.valueChanged.connect(self.slider_C_changed)
        self.ui.slice_L_SB.valueChanged.connect(self.slider_L_changed)
        self.ui.slice_R_SB.valueChanged.connect(self.slider_R_changed)

        # # DATA ---------------------------------------------
        # if datap1 is not None:
        #     self.data_1 = Data.Data()
        #     self.data_1.create_data(datap1, 'datap1')
        #     self.params['voxel_size'] = datap1['voxelsize_mm']
        # else:
        #     self.data_1 = None
        # if datap2 is not None:
        #     self.data_2 = Data.Data()
        #     self.data_2.create_data(datap2, 'datap2')
        # else:
        #     self.data_2 = None
        # #--------------------------------------------------

        # if self.data_1.loaded:
        #     self.hist_widget = Hist_widget(data=self.data_1.data[np.nonzero(self.data_1.mask)])
        # else:
        #     self.hist_widget = Hist_widget()
        # self.hist_widget.heal_parameter_changed.connect(self.update_models_from_widget)
        # self.hist_widget.hypo_parameter_changed.connect(self.update_models_from_widget)
        # self.hist_widget.hyper_parameter_changed.connect(self.update_models_from_widget)

        # self.area_hist_widget = ahw.AreaHistWidget()

        # computational core
        ## self.coco = Computational_core.Computational_core(fname, self.params, self.statusBar())
        # if self.data_1.loaded:
        #     self.ui.figure_L_CB.addItem(self.data_1.filename.split('/')[-1])
        #     self.ui.figure_R_CB.addItem(self.data_1.filename.split('/')[-1])
        #     self.data_L = self.data_1
        #     self.active_data_idx = 1
        #     self.active_data = self.data_1
        #     if not self.data_2.loaded:
        #         self.data_R = self.data_1
        # if self.data_2.loaded:
        #     # self.ui.serie_2_RB.setText('Serie #2: ' + self.cc.data_2.filename.split('/')[-1])
        #     self.ui.figure_L_CB.addItem(self.data_2.filename.split('/')[-1])
        #     self.ui.figure_R_CB.addItem(self.data_2.filename.split('/')[-1])
        #     self.data_R = self.data_2
        #     self.ui.figure_R_CB.setCurrentIndex(1)
        #     if not self.data_1.loaded:
        #         self.data_L = self.data_2
        #         self.active_data = self.data_2
        #         self.active_data_idx = 2

        # seting up the range of the scrollbar to cope with the number of slices
        # if self.active_data_idx == 1:
        #     self.ui.slice_C_SB.setMaximum(self.data_1.n_slices - 1)
        #     self.ui.slice_L_SB.setMaximum(self.data_1.n_slices - 1)
        # else:
        #     self.ui.slice_C_SB.setMaximum(self.data_2.n_slices - 1)
        #     self.ui.slice_L_SB.setMaximum(self.data_2.n_slices - 1)

        # if self.data_2.loaded:
        #     self.ui.slice_R_SB.setMaximum(self.data_2.n_slices - 1)

        # self.view_L = data_view_widget.SliceBox(self.data_L.data_aview.shape[:-1], self.voxel_size, self)
        # self.view_L.setCW(self.win_l, 'c')
        # self.view_L.setCW(self.win_w, 'w')
        # self.view_L.setSlice(self.data_L.data_aview[...,0])
        # # mouse click signal
        # self.view_L.mouseClickSignal.connect(self.mouse_click_event)
        # self.view_L.mousePressEvent = self.view_L.myMousePressEvent

        # self.view_R = data_view_widget.SliceBox(self.data_R.data_aview.shape[:-1], self.voxel_size, self)
        # self.view_R.setCW(self.win_l, 'c')
        # self.view_R.setCW(self.win_w, 'w')
        # self.view_R.setSlice(self.data_R.data_aview[...,0])
        # if not self.show_view_L:
        #     self.view_L.setVisible(False)
        # if not self.show_view_R:
        #     self.view_R.setVisible(False)

        # to be able to capture key press events immediately
        self.setFocus()

    def setup_data(self, datap1=None, datap2=None):
        # DATA ---------------------------------------------
        if datap1 is not None:
            self.data_1 = Data.Data()
            self.data_1.create_data(datap1, 'datap1')
            self.params['voxel_size'] = datap1['voxelsize_mm']
        if datap2 is not None:
            self.data_2 = Data.Data()
            self.data_2.create_data(datap2, 'datap2')

        if self.data_1.loaded:
            # self.hist_widget = Hist_widget(data=self.data_1.data[np.nonzero(self.data_1.mask)])
            self.hist_widget.set_data(data=self.data_1.data[np.nonzero(self.data_1.mask)])

            #seting up figure and data_L, data_R
            self.ui.figure_L_CB.addItem(self.data_1.filename.split('/')[-1])
            self.ui.figure_R_CB.addItem(self.data_1.filename.split('/')[-1])
            self.data_L = self.data_1
            self.active_data_idx = 1
            self.active_data = self.data_1
            if not self.data_2.loaded:
                self.data_R = self.data_1

            self.ui.slice_C_SB.setMaximum(self.data_1.n_slices - 1)
            self.ui.slice_L_SB.setMaximum(self.data_1.n_slices - 1)

        if self.data_2.loaded:
            # self.ui.serie_2_RB.setText('Serie #2: ' + self.cc.data_2.filename.split('/')[-1])
            self.ui.figure_L_CB.addItem(self.data_2.filename.split('/')[-1])
            self.ui.figure_R_CB.addItem(self.data_2.filename.split('/')[-1])
            self.data_R = self.data_2
            self.ui.figure_R_CB.setCurrentIndex(1)
            if not self.data_1.loaded:
                self.data_L = self.data_2
                self.active_data = self.data_2
                self.active_data_idx = 2

                self.hist_widget.set_data(data=self.data_2.data[np.nonzero(self.data_2.mask)])

                self.ui.slice_C_SB.setMaximum(self.data_2.n_slices - 1)
                self.ui.slice_L_SB.setMaximum(self.data_2.n_slices - 1)

            self.ui.slice_R_SB.setMaximum(self.data_2.n_slices - 1)

        # self.view_L = data_view_widget.SliceBox(self.data_L.data_aview.shape[:-1], self.voxel_size, self)
        self.view_L.setup_widget(self.data_L.data_aview.shape[:-1], self.params['voxel_size'][1:])
        self.view_L.setCW(self.params['win_l'], 'c')
        self.view_L.setCW(self.params['win_w'], 'w')
        self.view_L.setSlice(self.data_L.data_aview[...,0])
        # mouse click signal
        self.view_L.mouseClickSignal.connect(self.mouse_click_event)
        self.view_L.mousePressEvent = self.view_L.myMousePressEvent

        # self.view_R = data_view_widget.SliceBox(self.data_R.data_aview.shape[:-1], self.voxel_size, self)
        self.view_R.setup_widget(self.data_R.data_aview.shape[:-1], self.params['voxel_size'][1:])
        self.view_R.setCW(self.params['win_l'], 'c')
        self.view_R.setCW(self.params['win_w'], 'w')
        self.view_R.setSlice(self.data_R.data_aview[...,0])
        if not self.show_view_L:
            self.view_L.setVisible(False)
        if not self.show_view_R:
            self.view_R.setVisible(False)


    def keyPressEvent(self, QKeyEvent):
        print 'key event: ',
        key = QKeyEvent.key()
        if key == QtCore.Qt.Key_Escape:
            print 'Escape'
            if self.view_L.area_hist_widget is not None and self.view_L.area_hist_widget.isVisible():
                self.view_L.circle_active = False
                self.view_L.area_hist_widget.close()
                self.view_L.setMouseTracking(False)
                self.view_L.updateSlice()
            elif self.hist_widget is not None and self.hist_widget.isVisible():
                self.hist_widget.close()
            elif self.objects_widget is not None and self.objects_widget.isVisible():
                self.objects_widget.close()
            else:
                self.close()
        elif key == QtCore.Qt.Key_H:
            print 'H'
            self.action_show_color_model_callback()
        elif key == QtCore.Qt.Key_O:
            print 'O'
            self.action_show_object_list_callback()
        elif key == QtCore.Qt.Key_L:
            print 'L'
            self.run_callback()
        elif key == QtCore.Qt.Key_C:
            print 'C'
            self.action_circle_callback()
        elif key == QtCore.Qt.Key_A:  # interactively add an object
            print 'A'
            self.add_obj_BTN_callback()
        else:
            print key, ' - unrecognized hot key.'

    def mouse_click_event(self, coords, density):
        if self.mode == MODE_ADDING:
            self.add_obj_event(coords, density)
        elif self.mode == MODE_VIEWING:
            self.find_clicked_object(coords)

    def add_obj_BTN_callback(self):
        print 'adding object'
        self.mode = MODE_ADDING
        # self.view_L.mouseClickSignal.connect(self.add_obj_event)
        # self.view_L.mouseClickSignal.connect(self.add_obj_event)
        # self.view_L.mousePressEvent = self.view_L.myMousePressEvent

    def find_clicked_object(self, coords):
        print 'finding clicked object'
        lbl = self.active_data.objects[self.actual_slice_L, coords[1], coords[0]]
        # for i in self.objects_widget.ui.objects_TV. rowAt()
        #     self.objects_widget.ui.objects_TV.rowAt()
        if lbl > -1:
            for i in range(self.table_model.rowCount()):
                if self.table_model.data(self.table_model.index(i,0)).toInt()[0] == lbl:
                    break
            self.objects_widget.ui.objects_TV.selectRow(i)
            # self.objects_widget.ui.objects_TV.selectRow(lbl - 1)
        self.view_L.updateSlice()

    def add_obj_event(self, coords, density):
        self.mode = MODE_VIEWING
        print 'add_obj_event - coords: ', coords, ', density: ', density
        center = [self.actual_slice_L, coords[1], coords[0]]
        if self.data_L.objects is not None:
            idx = self.data_L.objects.max() + 1
        else:
            idx = 1
        new_les = Lesion.create_lesion_from_pt(center, density, idx)
        if self.active_data.models is not None and self.active_data.models['heal'].mean() < density:
            lbl = self.params['hyper_label']
        else:
            lbl = self.params['hypo_label']
        self.data_L.labels[center[0], center[2], center[1]] = lbl
        self.data_L.objects[center[0], center[2], center[1]] = idx

        self.data_L.append_lesion(new_les)

        # objects_labels = [x.label for x in self.active_data.lesions]
        filtered_idxs = coco.objects_filtration(self.active_data, self.params)
        # self.fill_table(self.data_L.lesions, self.data_L.labels)
        # self.fill_table(self.data_L.lesions, self.data_L.labels, self.data_L.labels_filt) #self.cc.filtered_idxs)
        self.fill_table(self.active_data.lesions, self.active_data.labels, filtered_idxs)
        # self.actual_data.labels_filt
        self.ui.show_contours_L_BTN.setEnabled(True)

        self.view_L.updateSlice()

    def min_compactness_SL_changed(self, value):
        self.objects_widget.ui.min_compactness_LE.setText('%.3f' % (value * self.params['compactness_step']))
        self.object_slider_changed(value)

    def min_compactness_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.objects_widget.ui.min_compactness_SL.setValue(int(value * self.objects_widget.ui.min_compactness_SL.maximum()))
            # self.params['compactness'] = value
        except:
            pass

    def action_circle_callback(self):
        self.view_L.circle_active = True
        self.view_L.setMouseTracking(True)
        self.view_L.area_hist_widget = ahw.AreaHistWidget()
        self.view_L.area_hist_widget.show()
        # self.view_R.circle_active = True

    def action_show_color_model_callback(self):
        if self.hist_widget.data is None and self.data_L.loaded:
            self.hist_widget.set_data(self.data_L.data.data[np.nonzero(self.data_L.mask)])
        self.hist_widget.show()
        self.hist_widget.setFocus()

    def action_show_object_list_callback(self):
        # if self.objects_widget is None:
        #     self.objects_widget = Objects_widget()
        #     self.objects_widget.area_RS.endValueChanged.connect(self.max_area_changed_callback)
        #     self.objects_widget.area_RS.startValueChanged.connect(self.min_area_changed_callback)
        self.objects_widget.show()
        self.objects_widget.setFocus()

    # def serie_1_RB_callback(self):
    #     self.active_data_idx = 1
    #     self.active_data = self.cc.data_1
    #
    # def serie_2_RB_callback(self):
    #     self.active_data_idx = 2
    #     self.active_data = self.cc.data_2

    def my_selection_changed(self):
        pass

    def selection_changed(self, selected, deselected):
        if self.objects_widget.isActiveWindow():
            if selected.indexes():
                selected_objects_labels = [self.table_model.objects[x.row()].label for x in self.objects_widget.ui.objects_TV.selectionModel().selectedRows()]
                # print 'show only', self.selected_objects_labels
                slice = [int(x.center[0]) for x in self.active_data.lesions if x.label == selected_objects_labels[0]][0]
                self.ui.slice_C_SB.setValue(slice)
            else:
                selected_objects_labels = [x.label for x in self.table_model.objects]
                # print 'show all', self.selected_objects_labels
            # min_area, max_area = self.objects_widget.area_RS.getRange()
            # min_density, max_density = self.objects_widget.density_RS.getRange()
            # min_comp = self.objects_widget.ui.min_compactness_SL.value() / self.params['compactness_step']
            # coco.objects_filtration(self.selected_objects_labels, self.params, area=(min_area, max_area),
            #                         density=(min_density, max_density), compactness=min_comp)
            coco.objects_filtration(self.active_data, self.params, selected_labels=selected_objects_labels)
        self.update_view_L()
        self.update_view_R()
        # self.activateWindow()
        # self.objects_widget.activateWindow()

    def object_slider_changed(self, value):
        min_area = self.objects_widget.area_RS.start()
        max_area = self.objects_widget.area_RS.end()
        min_density = self.objects_widget.density_RS.start()
        max_density = self.objects_widget.density_RS.end()
        min_comp = self.objects_widget.ui.min_compactness_SL.value() * self.params['compactness_step']
        self.params['max_area'] = max_area
        self.params['min_area'] = min_area
        self.params['max_density'] = max_density
        self.params['min_density'] = min_density
        self.params['min_compactness'] = min_comp

        # self.selected_objects_labels
        # objects_labels = [x.label for x in self.active_data.lesions]
        filtered_idxs = coco.objects_filtration(self.active_data, self.params)
        if filtered_idxs is not None:
            self.fill_table(self.active_data.lesions, self.active_data.labels, filtered_idxs)
            # # TODO: nasleduje prasarna
            # if self.view_L.show_mode == self.view_L.SHOW_LABELS:
            #     self.show_labels_L_callback()
            # if self.view_R.show_mode == self.view_R.SHOW_LABELS:
            #     self.show_labels_R_callback()
            self.update_view_L()
            self.update_view_R()

    def load_parameters(self, config_path='config.ini'):
        config = ConfigParser.ConfigParser()
        config.read(config_path)

        params = dict()

        # an automatic way
        for section in config.sections():
            for option in config.options(section):
                try:
                    params[option] = config.getint(section, option)
                except ValueError:
                    try:
                        params[option] = config.getfloat(section, option)
                    except ValueError:
                        if option == 'voxel_size':
                            str = config.get(section, option)
                            params[option] = np.array(map(int, str.split(', ')))
                        else:
                            params[option] = config.get(section, option)

        return params

    def scroll_event(self, value, who):
        if who == 0:  # left viewer
            new = self.actual_slice_L + value
            if (new < 0) or (new >= self.data_L.n_slices):
                return
        elif who == 1:  # right viewer
            new = self.actual_slice_R + value
            if (new < 0) or (new >= self.data_R.n_slices):
                return

    def slider_C_changed(self, val):
        if val == self.actual_slice_L:
            return

        if (val >= 0) and (val < self.data_L.n_slices):
            diff = val - self.actual_slice_L
            self.actual_slice_L = val
        else:
            return

        new_slice_R = self.actual_slice_R + diff
        if new_slice_R < 0:
            new_slice_R = 0
        elif new_slice_R >= self.data_R.n_slices:
            new_slice_R = self.data_R.n_slices - 1

        self.actual_slice_R = new_slice_R

        self.ui.slice_L_SB.setValue(self.actual_slice_L)
        self.ui.slice_R_SB.setValue(self.actual_slice_R)

        im_L = self.get_image('L')
        im_R = self.get_image('R')
        if self.show_mode_L == SHOW_CONTOURS:
            labels_L = self.data_L.labels_filt[self.actual_slice_L, :, :]
            obj_centers_L = self.data_L.object_centers_filt[self.actual_slice_L, ...]
        else:
            labels_L = None
            obj_centers_L = None
        if self.show_mode_R == SHOW_CONTOURS:
            labels_R = self.data_R.labels_filt[self.actual_slice_R, :, :]
            obj_centers_R = self.data_R.object_centers_filt[self.actual_slice_R, ...]
        else:
            labels_R = None
            obj_centers_R = None

        self.view_L.setSlice(im_L, contours=labels_L, centers=obj_centers_L)
        self.view_R.setSlice(im_R, contours=labels_R, centers=obj_centers_R)

        self.ui.slice_number_L_LBL.setText('%i/%i' % (self.actual_slice_L + 1, self.data_L.n_slices))
        self.ui.slice_number_C_LBL.setText('slice # = %i/%i' % (self.actual_slice_L + 1, self.data_L.n_slices))

    def slider_L_changed(self, val):
        if val == self.actual_slice_L:
            return

        if (val >= 0) and (val < self.data_L.n_slices):
            self.actual_slice_L = val
        else:
            return

        self.ui.slice_C_SB.setValue(self.actual_slice_L)

        im_L = self.get_image('L')
        if self.show_mode_L == SHOW_CONTOURS:
            labels_L = self.data_L.labels_filt[self.actual_slice_L, :, :]
            obj_centers = self.data_L.object_centers_filt[self.actual_slice_L, ...]
        else:
            labels_L = None
            obj_centers = None

        self.view_L.setSlice(im_L, contours=labels_L, centers=obj_centers)

        self.ui.slice_number_L_LBL.setText('%i/%i' % (self.actual_slice_L + 1, self.data_L.n_slices))
        self.ui.slice_number_C_LBL.setText('slice # = %i/%i' % (self.actual_slice_L + 1, self.data_L.n_slices))

    def slider_R_changed(self, val):
        if (val >= 0) and (val < self.data_R.n_slices):
            self.actual_slice_R = val
        else:
            return

        self.ui.slice_number_R_LBL.setText('%i/%i' % (self.actual_slice_R + 1, self.data_R.n_slices))

        im_R = self.get_image('R')
        if self.show_mode_R == SHOW_CONTOURS:
            labels_R = self.data_R.labels_filt[self.actual_slice_R, :, :]
            obj_centers = self.data_R.object_centers_filt[self.actual_slice_R, ...]
        else:
            labels_R = None
            obj_centers = None

        self.view_R.setSlice(im_R, contours=labels_R, centers=obj_centers)

    def calculate_models_callback(self):
        self.statusBar().showMessage('Calculating intensity models...')
        if self.params['zoom']:
            data = self.data_zoom(self.active_data.data, self.active_data.voxel_size, self.params['working_voxel_size_mm'])
            mask = self.data_zoom(self.active_data.mask, self.active_data.voxel_size, self.params['working_voxel_size_mm'])
        else:
            data = tools.resize3D(self.active_data.data, self.params['scale'], sliceId=0)
            mask = tools.resize3D(self.active_data.mask, self.params['scale'], sliceId=0)
        self.active_data.models = coco.calculate_intensity_models(data, mask, self.params)
        self.statusBar().showMessage('Intensity models calculated.')
        self.update_models()

    def update_models(self):
        if self.hist_widget is None:
            self.hist_widget = Hist_widget(data=self.data_L.data)
        self.hist_widget.set_models(self.active_data.models)
        # self.hist_widget.update_heal_rv(self.active_data.models['heal'])
        # self.hist_widget.update_hypo_rv(self.active_data.models['hypo'])
        # self.hist_widget.update_hyper_rv(self.active_data.models['hyper'])

        # self.hist_widget.update_figures()

    def update_models_from_widget(self, mean, std, key):
        # key = 'heal'
        # print key, self.active_data.models[key].mean(),
        self.active_data.models[key] = scista.norm(mean, std)
        # print self.active_data.models[key].mean()

    def run_callback(self):
        # Viewer_3D.run(self.data)
        # viewer = Viewer_3D.Viewer_3D(self.data)
        # viewer.show()

        # run localization
        self.statusBar().showMessage('Localization started...')
        coco.run_mrf(self.active_data, self.params)
        # self.active_data, self.models = coco.run_mrf(self.active_data, self.params, models=self.models)
        self.update_models()

        # seting up range of area slider
        areas = [x.area for x in self.active_data.lesions]
        self.objects_widget.set_area_range(areas)

        densities = [x.mean_density for x in self.active_data.lesions]
        self.objects_widget.set_density_range(densities)

        # filling table with objects
        # objects_labels = [x.label for x in self.active_data.lesions]
        # self.fill_table(self.active_data.lesions, self.active_data.objects, objects_labels)

        self.ui.show_labels_L_BTN.setEnabled(True)
        self.ui.show_contours_L_BTN.setEnabled(True)

    def fill_table(self, lesions, labels, idxs=None):
        # if self.objects_widget is None:
        #     self.objects_widget = Objects_widget()
        #     # tableview selection changed
        #     self.objects_widget.ui.objects_TV.selectionModel().selectionChanged.connect(self.selection_changed)
        if idxs is None:
            idxs = [x.label for x in lesions]
            # lesions_filtered = lesions[:]  # copying the list
        # else:
        lesions_filtered = [x for x in lesions if x.label in idxs]
        # labels_filtered = np.where(labels in idxs, labels, 0)
        labels_filtered = labels * np.in1d(labels, idxs).reshape(labels.shape)
        self.table_model = mtm.MyTableModel(lesions_filtered, labels_filtered)
        self.objects_widget.ui.objects_TV.setModel(self.table_model)
        self.objects_widget.ui.objects_TV.selectionModel().selectionChanged.connect(self.selection_changed)

        # self.ui.objects_TV.selectionModel().selectionChanged.connect(self.selection_changed)

    # def fill_table(self, labels, areas, comps):
    #     features = np.vstack((labels, areas, comps)).T
    #     self.table_model = mtm.MyTableModel(features)
    #     self.ui.objects_TV.setModel(self.table_model)
    #     self.ui.objects_TV.selectionModel().selectionChanged.connect(self.selection_changed)
    #     # self.tableview.verticalHeader().setVisible(True)


    def view_L_callback(self):
        self.show_view_L = not self.show_view_L
        self.view_L.setVisible(self.show_view_L)

        # enabling and disabling other toolbar icons
        self.ui.show_im_L_BTN.setEnabled(self.show_view_L)

        if self.show_view_L and self.data_L.labels is not None:
            self.ui.show_labels_L_BTN.setEnabled(True)
            self.ui.show_contours_L_BTN.setEnabled(True)
        else:
            self.ui.show_labels_L_BTN.setEnabled(False)
            self.ui.show_contours_L_BTN.setEnabled(False)

        # if not self.show_view_L:
            # self.ui.viewer_F.setFixedWidth(self.ui.viewer_F.width()/2)
            # self.resize(self.height(), self.width() - self.ui.viewer_F.width()/2)

        self.statusBar().showMessage('Left view set to %s' % self.show_view_L)
        # print 'view_1 set to', self.show_view_1

        # self.form_widget.update_figures()
        self.view_L.update()

    def view_R_callback(self):
        self.show_view_R = not self.show_view_R
        self.view_R.setVisible(self.show_view_R)

        # enabling and disabling other toolbar icons
        self.ui.show_im_R_BTN.setEnabled(not self.ui.show_im_R_BTN.isEnabled())

        if self.show_view_R and self.data_R.labels is not None:
            self.ui.show_labels_R_BTN.setEnabled(True)
            self.ui.show_contours_R_BTN.setEnabled(True)
        else:
            self.ui.show_labels_R_BTN.setEnabled(False)
            self.ui.show_contours_R_BTN.setEnabled(False)

        # if not self.show_view_R:
        #     self.ui.viewer_F.setFixedWidth(self.ui.viewer_F.width()/2)

        self.statusBar().showMessage('Right view set to %s' % self.show_view_R)
        # print 'view_2 set to', self.show_view_2

        # self.form_widget.update_figures()
        self.view_R.update()

    def show_im_L_callback(self):
        self.show_mode_L = SHOW_IM
        self.view_L.show_mode = self.view_L.SHOW_IM

        im = self.get_image('L')
        self.view_L.setSlice(im)

        self.statusBar().showMessage('data_L set to im')

    def show_im_R_callback(self):
        self.show_mode_R = SHOW_IM
        self.view_R.show_mode = self.view_R.SHOW_IM

        im = self.get_image('R')
        self.view_R.setSlice(im)

        self.statusBar().showMessage('data_R set to im')

    def show_labels_L_callback(self):
        self.show_mode_L = SHOW_LABELS
        self.view_L.show_mode = self.view_L.SHOW_LABELS

        im = self.get_image('L')
        self.view_L.setSlice(im)

        self.statusBar().showMessage('data_L set to labels')

    def show_labels_R_callback(self):
        self.show_mode_R = SHOW_LABELS
        self.view_R.show_mode = self.view_R.SHOW_LABELS

        im = self.get_image('R')
        self.view_R.setSlice(im)

        self.statusBar().showMessage('data_R set to labels')

    def show_contours_L_callback(self):
        if self.show_mode_L == SHOW_CONTOURS:
            self.view_L.contours_mode_is_fill = not self.view_L.contours_mode_is_fill
        self.show_contours_L()

    def show_contours_L(self):
        self.show_mode_L = SHOW_CONTOURS
        self.view_L.show_mode = self.view_L.SHOW_CONTOURS

        im = self.get_image('L')
        labels = self.data_L.labels_filt[self.actual_slice_L, :, :]
        # obj_centers = [self.data_L.object_centers[:2] if self.data_L.object_centers[2] == self.actual_slice_L]
        # obj_centers = [x[1:] for x in self.data_L.object_centers if round(x[0]) == self.actual_slice_L]
        self.view_L.setSlice(im, contours=labels, centers=self.data_L.object_centers_filt[self.actual_slice_L,...])

        self.statusBar().showMessage('data_L set to contours')

    def show_contours_R_callback(self):
        if self.show_mode_R == SHOW_CONTOURS:
            self.view_R.contours_mode_is_fill = not self.view_R.contours_mode_is_fill
        self.show_contours_R()

    def show_contours_R(self):
        self.show_mode_R = SHOW_CONTOURS
        self.view_R.show_mode = self.view_R.SHOW_CONTOURS

        im = self.get_image('R')
        labels = self.data_R.labels_filt[self.actual_slice_R, :, :]
        self.view_R.setSlice(im, contours=labels, centers=self.data_R.object_centers_filt[self.actual_slice_R,...])

        self.statusBar().showMessage('data_R set to contours')

    def update_view_L(self):
        if self.show_view_L:
            if self.view_L.show_mode == self.view_L.SHOW_LABELS:
                self.show_labels_L_callback()
            elif self.view_L.show_mode == self.view_L.SHOW_CONTOURS:
                # self.show_contours_L_callback()
                self.show_contours_L()

    def update_view_R(self):
        if self.show_view_R:
            if self.view_R.show_mode == self.view_R.SHOW_LABELS:
                self.show_labels_R_callback()
            elif self.view_R.show_mode == self.view_R.SHOW_CONTOURS:
                # self.show_contours_R_callback()
                self.show_contours_R()

    def figure_L_CB_callback(self):
        if self.ui.figure_L_CB.currentIndex() == 0:
            self.data_L = self.data_1
        elif self.ui.figure_L_CB.currentIndex() == 1:
            self.data_L = self.data_2

        if self.actual_slice_L >= self.data_L.n_slices:
            self.actual_slice_L = self.data_L.n_slices - 1

        self.ui.slice_L_SB.setMaximum(self.data_L.n_slices - 1)
        self.ui.slice_C_SB.setMaximum(self.data_L.n_slices - 1)

        if (self.data_L.labels is not None) and self.show_view_L:
            self.ui.show_labels_L_BTN.setEnabled(True)
            self.ui.show_contours_L_BTN.setEnabled(True)
        else:
            self.ui.show_labels_L_BTN.setEnabled(False)
            self.ui.show_contours_L_BTN.setEnabled(False)

        self.view_L.reinit((self.data_L.shape[2], self.data_L.shape[1]))
        self.show_im_L_callback()

    def figure_R_CB_callback(self):
        if self.ui.figure_R_CB.currentIndex() == 0:
            self.data_R = self.data_1
        elif self.ui.figure_R_CB.currentIndex() == 1:
            self.data_R = self.data_2

        if self.actual_slice_R >= self.data_R.n_slices:
            self.actual_slice_R = self.data_R.n_slices - 1

        self.ui.slice_R_SB.setMaximum(self.data_R.n_slices - 1)

        if (self.data_R.labels is not None) and self.show_view_R:
            self.ui.show_labels_R_BTN.setEnabled(True)
            self.ui.show_contours_R_BTN.setEnabled(True)
        else:
            self.ui.show_labels_R_BTN.setEnabled(False)
            self.ui.show_contours_R_BTN.setEnabled(False)

        # self.view_R.reinit(self.data_R.shape[2:0:-1])
        self.view_R.reinit((self.data_R.shape[2], self.data_R.shape[1]))
        self.show_im_R_callback()

    def action_load_serie_callback(self, serie_number):
        # loading file and creating dataplus format
        fname = str(QtGui.QFileDialog.getOpenFileName(self, 'Open file', self.params['data_dir']))
        datap = dr.Get3DData(fname, dataplus_format=True)

        # creating data structure
        # data = Data.Data()
        # data.create_data(datap, fname)

        if serie_number == 1:
            self.setup_data(datap1=datap)
        else:
            self.setup_data(datap2=datap)
        # print 'Does not work yet.'
        # print fname

    def remove_obj_BTN_callback(self):
        indexes = self.objects_widget.ui.objects_TV.selectionModel().selectedRows()
        # for i in reversed(indexes):
        #     self.table_model.removeRow(i.row())
        for i in indexes:
            obj = self.table_model.objects[i.row()]
            self.remove_object(obj)

        # objects_labels = [x.label for x in self.active_data.lesions]
        filtered_idxs = coco.objects_filtration(self.active_data, self.params)
        self.fill_table(self.active_data.lesions, self.active_data.objects, filtered_idxs)
        # #TODO: nasleduje prasarna
        # if self.view_L.show_mode == self.view_L.SHOW_LABELS:
        #     self.show_labels_L_callback()
        # if self.view_R.show_mode == self.view_R.SHOW_LABELS:
        #     self.show_labels_R_callback()
        self.update_view_L()
        self.update_view_R()

    def remove_object(self, obj):
        lbl = obj.label
        im = self.active_data.objects == lbl
        self.active_data.objects[np.nonzero(im)] = self.params['bgd_label']
        self.active_data.labels[np.nonzero(im)] = self.params['healthy_label']

        self.active_data.lesions.remove(obj)
        print 'removed label', lbl

    def get_image(self, site):
        im = None
        if site == 'L':
            if self.show_mode_L == SHOW_IM or self.show_mode_L == SHOW_CONTOURS:
                im = self.data_L.data_aview[..., self.actual_slice_L]
            elif self.show_mode_L == SHOW_LABELS:
                # im = self.data_L.labels_aview[...,self.actual_slice_L]
                im = self.data_L.labels_filt_aview[..., self.actual_slice_L]
            # elif self.show_mode_L == SHOW_FILTERED_LABELS:
            #     im = self.data_L.labels_filt_aview[...,self.actual_slice_L]
        elif site == 'R':
            if self.show_mode_R == SHOW_IM or self.show_mode_R == SHOW_CONTOURS:
                im = self.data_R.data_aview[..., self.actual_slice_R]
            elif self.show_mode_R == SHOW_LABELS:
                # im = self.data_R.labels_aview[...,self.actual_slice_R]
                im = self.data_R.labels_filt_aview[..., self.actual_slice_R]
            # elif self.show_mode_R == SHOW_FILTERED_LABELS:
            #     im = self.data_R.labels_filt_aview[...,self.actual_slice_R]
        return im

    def run(self, im, labels, healthy_label, hypo_label, hyper_label, slice_axis=2, disp_smoothed=False):
        if slice_axis == 0:
            im = np.transpose(im, (1, 2, 0))
            labels = np.transpose(labels, (1, 2, 0))
        app = QtGui.QApplication(sys.argv)
        le = LessionEditor(im, labels, healthy_label, hypo_label, hyper_label, disp_smoothed)
        le.show()
        sys.exit(app.exec_())

################################################################################
################################################################################
if __name__ == '__main__':
    fname_1 = None
    fname_2 = None

    # 2 hypo, 1 on the border --------------------
    # TODO: study ID 29 - 3/3, 2 velke spoji v jeden
    # slice = 17
    fname_1 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_183_46324212_venous_5.0_B30f-.pklz'
    fname_2 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_183_46324212_arterial_5.0_B30f-.pklz'

    # hypo in venous -----------------------
    # arterial - bad
    # fname_1 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_186_49290986_venous_0.6_B20f-.pklz'
    # venous - good
    # fname_2 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_186_49290986_arterial_0.6_B20f-.pklz'


    # hyper, 1 on the border -------------------
    # arterial 0.6mm - not that bad
    # fname_1 = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_239_61293268_DE_Art_Abd_0.75_I26f_M_0.5-.pklz'
    # venous 5mm - bad
    # fname_2 = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_239_61293268_DE_Ven_Abd_0.75_I26f_M_0.5-.pklz'

    # shluk -----------------
    # arterial 5mm
    # TODO: study ID 18 - velke najde (hyper v arterialni fazi), 1/2 z malych
    # fname_1 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_180_49509315_venous_5.0_B30f-.pklz'
    # fname_2 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_180_49509315_arterial_5.0_B30f-.pklz'


    # targeted
    # arterial 0.6mm - bad
    # fname_1 = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_238_54280551_Abd_Arterial_0.75_I26f_3-.pklz'
    # venous 0.6mm - bad
    # fname_2 = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_238_54280551_Abd_Venous_0.75_I26f_3-.pklz'

    # TODO: study ID 25 - 2/2
    # fname_1 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_185_48441644_venous_5.0_B30f-.pklz'
    # fname_2 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_185_48441644_arterial_5.0_B30f-.pklz'

    # TODO: study ID 21 - 0/2, nenasel ani jeden pekny hypo
    # slice = 6
    # fname_1 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_186_49290986_venous_5.0_B30f-.pklz'
    # fname_2 = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_186_49290986_arterial_5.0_B30f-.pklz'

    # runing application -------------------------
    # reading data
    dr = io3d.DataReader()
    if fname_1 is not None:
        datap_1 = dr.Get3DData(fname_1, dataplus_format=True)
    else:
        datap_1 = {'data3d': None, 'segmentation':None, 'slab':None, 'voxelsize_mm':None}
    if fname_2 is not None:
        datap_2 = dr.Get3DData(fname_2, dataplus_format=True)
    else:
        datap_2 = {'data3d': None, 'segmentation':None, 'slab':None, 'voxelsize_mm':None}

    # starting application
    app = QtGui.QApplication(sys.argv)
    le = LessionEditor(datap1=datap_1)#, datap2=datap_2)
    le.show()
    sys.exit(app.exec_())



__author__ = 'tomas'

import sys
from PyQt4 import QtGui, QtCore
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt
import numpy as np
import skimage.exposure as skiexp
import scipy.stats as scista

# Main widget containing figures etc
class Hist_widget(QtGui.QWidget):

    def __init__(self, window, cc):

        self.win = window  # link to the main window
        # self.data = self.win.data  # input data
        # self.mask = self.win.mask
        self.cc = cc
        self.data = self.cc.data_1.data  # input data
        self.mask = self.cc.data_1.mask
        # self.labels = self.win.labels  # input labeling
        # self.actual_slice = 0  # index of current data slice
        # self.n_slices = self.im.shape[2]  # numer of slices
        # self.healthy_label = self.win.healthy_label
        # self.hypo_label = self.win.hypo_label
        # self.hyper_label = self.win.hyper_label
        self.rv_healthy = None
        self.rv_hypo = None
        self.rv_hyper = None

        super(Hist_widget, self).__init__()
        self.init_UI_form()

    def init_UI_form(self):
        self.figure = plt.figure()
        self.axes = self.figure.add_axes([0, 0, 1, 1])

        self.canvas = FigureCanvas(self.figure)

        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.canvas, 1)
        self.setLayout(layout)

        # compute histogram
        ints = self.data[np.nonzero(self.mask)]
        self.hist, self.bins = skiexp.histogram(ints, nbins=256)

        self.update_figures()

    def update_heal_rv(self, new_rv):
        self.rv_healthy = new_rv

    def update_hypo_rv(self, new_rv):
        self.rv_hypo = new_rv

    def update_hyper_rv(self, new_rv):
        self.rv_hyper = new_rv

    def update_figures(self):
        plt.figure(self.figure.number)
        x = np.arange(0, 256, 0.1)  # artificial x-axis
        # self.figure.gca().cla()  # clearing the figure, just to be sure

        # plt.subplot(411)
        plt.plot(self.bins, self.hist, 'k')
        plt.hold(True)
        if self.rv_healthy and self.rv_hypo and self.rv_hyper:
            healthy_y = self.rv_healthy.pdf(x)
            if self.win.params['unaries_as_cdf']:
                hypo_y = (1 - self.rv_hypo.cdf(x)) * self.rv_healthy.pdf(self.rv_healthy.mean())
                hyper_y = self.rv_hyper.cdf(x) * self.rv_healthy.pdf(self.rv_healthy.mean())
            else:
                hypo_y = self.rv_hypo.pdf(x)
                hyper_y = self.rv_hyper.pdf(x)
            y_max = max(healthy_y.max(), hypo_y.max(), hyper_y.max())
            fac = self.hist.max() / y_max

            plt.plot(x, fac * healthy_y, 'g', linewidth=2)
            plt.plot(x, fac * hypo_y, 'b', linewidth=2)
            plt.plot(x, fac * hyper_y, 'r', linewidth=2)
            plt.title('all PDFs')
        ax = plt.axis()
        plt.axis([0, 256, ax[2], ax[3]])
        plt.hold(False)

        self.canvas.draw()


__author__ = 'tomas'

from sklearn.cluster import KMeans
import numpy as np


def generate_bootstrap_pairs(data, B, n_observations=1):
    n = data.size
    bpairs_idxs = np.random.randint(0, n, (2, np.round(n_observations * n).astype(np.int), B))
    bpairs_dat = data[np.unravel_index(bpairs_idxs.flatten(), data.shape)].reshape(bpairs_idxs.shape)

    # for i in range(bpairs.shape[2]):
    #     print bpairs_idxs[:,:,i]

    return bpairs_idxs, bpairs_dat


def classify(bpairs_dat, k):
    B = bpairs_dat.shape[2]
    kmeans = KMeans(k)
    kmeans.fit(bpairs_dat)
    labels = kmeans.labels_

    return labels


def calculate_clustering_distance():



def run(data, B, n_observations, k_min, k_max):
    n_k = k_max - k_min + 1
    instabs = np.zeros()
    for k in range(k_min, k_max + 1):
        # step 1 - generating B bootstrap sample-pairs
        bpairs_idxs, bpairs_dat = generate_bootstrap_pairs(data, B, n_observations)

        # step 2 - determine the total number of (n * 2) clusterings
        # labels = classify(bpairs_dat, k)
        clusterings = classify(bpairs_dat, k)

        # step 3 - calculate empirical clustering distance
        calculate_clustering_distance()


if __name__ == '__main__':
    k_min = 2
    k_max = 5
    n = 10
    B = 5
    n_observations = 1
    run(data, B, n_observations, k_min, k_max)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'objects_widget_GUI.ui'
#
# Created: Fri Sep 25 14:16:10 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName(_fromUtf8("Form"))
        Form.resize(481, 839)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        Form.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.verticalLayout = QtGui.QVBoxLayout(Form)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.area_layout = QtGui.QHBoxLayout()
        self.area_layout.setObjectName(_fromUtf8("area_layout"))
        self.label = QtGui.QLabel(Form)
        self.label.setObjectName(_fromUtf8("label"))
        self.area_layout.addWidget(self.label)
        self.verticalLayout.addLayout(self.area_layout)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.label_2 = QtGui.QLabel(Form)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.horizontalLayout_2.addWidget(self.label_2)
        self.min_compactness_LE = QtGui.QLineEdit(Form)
        self.min_compactness_LE.setMaximumSize(QtCore.QSize(50, 16777215))
        self.min_compactness_LE.setObjectName(_fromUtf8("min_compactness_LE"))
        self.horizontalLayout_2.addWidget(self.min_compactness_LE)
        self.min_compactness_SL = QtGui.QSlider(Form)
        self.min_compactness_SL.setMaximum(1000)
        self.min_compactness_SL.setSingleStep(1)
        self.min_compactness_SL.setOrientation(QtCore.Qt.Horizontal)
        self.min_compactness_SL.setObjectName(_fromUtf8("min_compactness_SL"))
        self.horizontalLayout_2.addWidget(self.min_compactness_SL)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.density_layout = QtGui.QHBoxLayout()
        self.density_layout.setObjectName(_fromUtf8("density_layout"))
        self.label_3 = QtGui.QLabel(Form)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.density_layout.addWidget(self.label_3)
        self.verticalLayout.addLayout(self.density_layout)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.crop_objects_BTN = QtGui.QPushButton(Form)
        self.crop_objects_BTN.setObjectName(_fromUtf8("crop_objects_BTN"))
        self.horizontalLayout.addWidget(self.crop_objects_BTN)
        self.remove_obj_BTN = QtGui.QPushButton(Form)
        self.remove_obj_BTN.setObjectName(_fromUtf8("remove_obj_BTN"))
        self.horizontalLayout.addWidget(self.remove_obj_BTN)
        self.add_obj_BTN = QtGui.QPushButton(Form)
        self.add_obj_BTN.setObjectName(_fromUtf8("add_obj_BTN"))
        self.horizontalLayout.addWidget(self.add_obj_BTN)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.objects_TV = QtGui.QTableView(Form)
        self.objects_TV.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.objects_TV.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.objects_TV.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.objects_TV.setObjectName(_fromUtf8("objects_TV"))
        self.verticalLayout.addWidget(self.objects_TV)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(_translate("Form", "Objects", None))
        self.label.setText(_translate("Form", "area [ml]:", None))
        self.label_2.setText(_translate("Form", "min. compactness=", None))
        self.label_3.setText(_translate("Form", "density [HU]:", None))
        self.crop_objects_BTN.setText(_translate("Form", "Crop", None))
        self.remove_obj_BTN.setText(_translate("Form", "Remove", None))
        self.add_obj_BTN.setText(_translate("Form", "Add", None))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'hist_widget_GUI.ui'
#
# Created: Wed Sep 16 07:24:34 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName(_fromUtf8("Form"))
        Form.resize(528, 574)
        Form.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.verticalLayout = QtGui.QVBoxLayout(Form)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.histogram_F = QtGui.QFrame(Form)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.histogram_F.sizePolicy().hasHeightForWidth())
        self.histogram_F.setSizePolicy(sizePolicy)
        self.histogram_F.setMinimumSize(QtCore.QSize(0, 100))
        self.histogram_F.setFrameShape(QtGui.QFrame.Box)
        self.histogram_F.setObjectName(_fromUtf8("histogram_F"))
        self.verticalLayout.addWidget(self.histogram_F)
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.frame = QtGui.QFrame(Form)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setMinimumSize(QtCore.QSize(0, 50))
        self.frame.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtGui.QFrame.Raised)
        self.frame.setObjectName(_fromUtf8("frame"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.frame)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label_12 = QtGui.QLabel(self.frame)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_12.sizePolicy().hasHeightForWidth())
        self.label_12.setSizePolicy(sizePolicy)
        self.label_12.setMinimumSize(QtCore.QSize(0, 0))
        self.label_12.setMaximumSize(QtCore.QSize(90, 16777215))
        self.label_12.setTextFormat(QtCore.Qt.RichText)
        self.label_12.setObjectName(_fromUtf8("label_12"))
        self.horizontalLayout.addWidget(self.label_12)
        self.label = QtGui.QLabel(self.frame)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setTextFormat(QtCore.Qt.RichText)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout.addWidget(self.label)
        self.hypo_mean_LE = QtGui.QLineEdit(self.frame)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hypo_mean_LE.sizePolicy().hasHeightForWidth())
        self.hypo_mean_LE.setSizePolicy(sizePolicy)
        self.hypo_mean_LE.setMaximumSize(QtCore.QSize(50, 16777215))
        self.hypo_mean_LE.setObjectName(_fromUtf8("hypo_mean_LE"))
        self.horizontalLayout.addWidget(self.hypo_mean_LE)
        self.hypo_mean_SL = QtGui.QSlider(self.frame)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hypo_mean_SL.sizePolicy().hasHeightForWidth())
        self.hypo_mean_SL.setSizePolicy(sizePolicy)
        self.hypo_mean_SL.setMaximum(100)
        self.hypo_mean_SL.setProperty("value", 0)
        self.hypo_mean_SL.setOrientation(QtCore.Qt.Horizontal)
        self.hypo_mean_SL.setObjectName(_fromUtf8("hypo_mean_SL"))
        self.horizontalLayout.addWidget(self.hypo_mean_SL)
        self.label_2 = QtGui.QLabel(self.frame)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.horizontalLayout.addWidget(self.label_2)
        self.hypo_std_SB = QtGui.QSpinBox(self.frame)
        self.hypo_std_SB.setMinimum(1)
        self.hypo_std_SB.setObjectName(_fromUtf8("hypo_std_SB"))
        self.horizontalLayout.addWidget(self.hypo_std_SB)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addWidget(self.frame)
        self.frame_2 = QtGui.QFrame(Form)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setMinimumSize(QtCore.QSize(0, 50))
        self.frame_2.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtGui.QFrame.Raised)
        self.frame_2.setObjectName(_fromUtf8("frame_2"))
        self.verticalLayout_7 = QtGui.QVBoxLayout(self.frame_2)
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.label_13 = QtGui.QLabel(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_13.sizePolicy().hasHeightForWidth())
        self.label_13.setSizePolicy(sizePolicy)
        self.label_13.setMaximumSize(QtCore.QSize(90, 16777215))
        self.label_13.setObjectName(_fromUtf8("label_13"))
        self.horizontalLayout_3.addWidget(self.label_13)
        self.label_14 = QtGui.QLabel(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_14.sizePolicy().hasHeightForWidth())
        self.label_14.setSizePolicy(sizePolicy)
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.horizontalLayout_3.addWidget(self.label_14)
        self.heal_mean_LE = QtGui.QLineEdit(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.heal_mean_LE.sizePolicy().hasHeightForWidth())
        self.heal_mean_LE.setSizePolicy(sizePolicy)
        self.heal_mean_LE.setMaximumSize(QtCore.QSize(50, 16777215))
        self.heal_mean_LE.setObjectName(_fromUtf8("heal_mean_LE"))
        self.horizontalLayout_3.addWidget(self.heal_mean_LE)
        self.heal_mean_SL = QtGui.QSlider(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.heal_mean_SL.sizePolicy().hasHeightForWidth())
        self.heal_mean_SL.setSizePolicy(sizePolicy)
        self.heal_mean_SL.setMaximum(100)
        self.heal_mean_SL.setProperty("value", 0)
        self.heal_mean_SL.setOrientation(QtCore.Qt.Horizontal)
        self.heal_mean_SL.setObjectName(_fromUtf8("heal_mean_SL"))
        self.horizontalLayout_3.addWidget(self.heal_mean_SL)
        self.label_15 = QtGui.QLabel(self.frame_2)
        self.label_15.setObjectName(_fromUtf8("label_15"))
        self.horizontalLayout_3.addWidget(self.label_15)
        self.heal_std_SB = QtGui.QSpinBox(self.frame_2)
        self.heal_std_SB.setMinimum(1)
        self.heal_std_SB.setObjectName(_fromUtf8("heal_std_SB"))
        self.horizontalLayout_3.addWidget(self.heal_std_SB)
        self.verticalLayout_7.addLayout(self.horizontalLayout_3)
        self.verticalLayout_2.addWidget(self.frame_2)
        self.frame_6 = QtGui.QFrame(Form)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_6.sizePolicy().hasHeightForWidth())
        self.frame_6.setSizePolicy(sizePolicy)
        self.frame_6.setMinimumSize(QtCore.QSize(0, 50))
        self.frame_6.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame_6.setFrameShadow(QtGui.QFrame.Raised)
        self.frame_6.setObjectName(_fromUtf8("frame_6"))
        self.verticalLayout_8 = QtGui.QVBoxLayout(self.frame_6)
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        self.horizontalLayout_10 = QtGui.QHBoxLayout()
        self.horizontalLayout_10.setObjectName(_fromUtf8("horizontalLayout_10"))
        self.label_16 = QtGui.QLabel(self.frame_6)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_16.sizePolicy().hasHeightForWidth())
        self.label_16.setSizePolicy(sizePolicy)
        self.label_16.setMaximumSize(QtCore.QSize(90, 16777215))
        self.label_16.setObjectName(_fromUtf8("label_16"))
        self.horizontalLayout_10.addWidget(self.label_16)
        self.label_17 = QtGui.QLabel(self.frame_6)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_17.sizePolicy().hasHeightForWidth())
        self.label_17.setSizePolicy(sizePolicy)
        self.label_17.setObjectName(_fromUtf8("label_17"))
        self.horizontalLayout_10.addWidget(self.label_17)
        self.hyper_mean_LE = QtGui.QLineEdit(self.frame_6)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hyper_mean_LE.sizePolicy().hasHeightForWidth())
        self.hyper_mean_LE.setSizePolicy(sizePolicy)
        self.hyper_mean_LE.setMaximumSize(QtCore.QSize(50, 16777215))
        self.hyper_mean_LE.setObjectName(_fromUtf8("hyper_mean_LE"))
        self.horizontalLayout_10.addWidget(self.hyper_mean_LE)
        self.hyper_mean_SL = QtGui.QSlider(self.frame_6)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hyper_mean_SL.sizePolicy().hasHeightForWidth())
        self.hyper_mean_SL.setSizePolicy(sizePolicy)
        self.hyper_mean_SL.setMaximum(100)
        self.hyper_mean_SL.setProperty("value", 0)
        self.hyper_mean_SL.setOrientation(QtCore.Qt.Horizontal)
        self.hyper_mean_SL.setObjectName(_fromUtf8("hyper_mean_SL"))
        self.horizontalLayout_10.addWidget(self.hyper_mean_SL)
        self.label_18 = QtGui.QLabel(self.frame_6)
        self.label_18.setObjectName(_fromUtf8("label_18"))
        self.horizontalLayout_10.addWidget(self.label_18)
        self.hyper_std_SB = QtGui.QSpinBox(self.frame_6)
        self.hyper_std_SB.setMinimum(1)
        self.hyper_std_SB.setObjectName(_fromUtf8("hyper_std_SB"))
        self.horizontalLayout_10.addWidget(self.hyper_std_SB)
        self.verticalLayout_8.addLayout(self.horizontalLayout_10)
        self.verticalLayout_2.addWidget(self.frame_6)
        self.verticalLayout.addLayout(self.verticalLayout_2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(_translate("Form", "Form", None))
        self.label_12.setText(_translate("Form", "<b>Hypodense:</b>", None))
        self.label.setText(_translate("Form", "mean=", None))
        self.hypo_mean_LE.setText(_translate("Form", "0", None))
        self.label_2.setText(_translate("Form", "std=", None))
        self.label_13.setText(_translate("Form", "<b>Healthy:</b>", None))
        self.label_14.setText(_translate("Form", "mean=", None))
        self.heal_mean_LE.setText(_translate("Form", "0", None))
        self.label_15.setText(_translate("Form", "std=", None))
        self.label_16.setText(_translate("Form", "<b>Hyperdense:</b>", None))
        self.label_17.setText(_translate("Form", "mean=", None))
        self.hyper_mean_LE.setText(_translate("Form", "0", None))
        self.label_18.setText(_translate("Form", "std=", None))




__author__ = 'tomas'

import sys
from PyQt4 import QtGui, QtCore

import numpy as np
import matplotlib.pyplot as plt
import scipy.ndimage.morphology as scindimor
import scipy.ndimage.measurements as scindimea
import scipy.ndimage.interpolation as scindiint
import scipy.ndimage as scindi
import scipy.stats as scista

import skimage.segmentation as skiseg
import skimage.morphology as skimor
import skimage.filter as skifil
import skimage.exposure as skiexp
import skimage.measure as skimea
import skimage.transform as skitra

# import cv2
# import pygco

import tools
import py3DSeedEditor
# from mayavi import mlab

import TumorVisualiser
import Viewer_3D

from sklearn import metrics
from sklearn.cluster import KMeans

import pickle

import ConfigParser

import logging
logger = logging.getLogger(__name__)
logging.basicConfig()


from lession_editor_GUI_slim import Ui_MainWindow
import Form_widget
import hist_widget_old
import My_table_model as mtm
import area_hist_widget as ahw

import Computational_core

import data_view_widget

# constants definition
SHOW_IM = 0
SHOW_LABELS = 1
SHOW_CONTOURS = 2

class Lession_editor(QtGui.QMainWindow):
    """Main class of the programm."""

    def __init__(self, fname, disp_smoothed=False, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        # uprava stylu pro lepsi vizualizaci splitteru
        QtGui.QApplication.setStyle(QtGui.QStyleFactory.create('Cleanlooks'))

        # self.im = im
        # self.labels = labels
        self.show_view_L = True
        self.show_view_R = False
        # self.healthy_label = healthy_label
        # self.hypo_label = hypo_label
        # self.hyper_label = hyper_label
        self.disp_smoothed = disp_smoothed
        # self.view_L_curr_idx = 0
        # self.view_R_curr_idx = 0

        self.show_mode_L = SHOW_IM
        self.show_mode_R = SHOW_IM

        # load parameters
        self.params = self.load_parameters()
        self.win_l = self.params['win_level']
        self.win_w = self.params['win_width']

        self.data_L = None
        self.data_R = None
        self.actual_slice_L = 0
        self.actual_slice_R = 0

        self.selected_objects_labels = None  # list of objects' labels selected in tableview

        # fill parameters to widgets
        self.fill_parameters()

        self.voxel_size = self.params['voxel_size']
        self.view_widget_width = 50
        self.two_views = False

        # self.area_hist_widget = ahw.AreaHistWidget()

        # computational core
        self.cc = Computational_core.Computational_core(fname, self.params, self.statusBar())
        if self.cc.data_1.loaded:
            # self.ui.serie_1_RB.setText('Serie #1: ' + self.cc.data_1.filename.split('/')[-1])
            # self.ui.figure_L_CB.addItem(self.cc.data_1.filename.split('/')[-1])
            # self.ui.figure_R_CB.addItem(self.cc.data_1.filename.split('/')[-1])
            self.data_L = self.cc.data_1
            if not self.cc.data_2.loaded:
                self.data_R = self.cc.data_1
        if self.cc.data_2.loaded:
            # self.ui.serie_2_RB.setText('Serie #2: ' + self.cc.data_2.filename.split('/')[-1])
            # self.ui.figure_L_CB.addItem(self.cc.data_2.filename.split('/')[-1])
            # self.ui.figure_R_CB.addItem(self.cc.data_2.filename.split('/')[-1])
            self.data_R = self.cc.data_2
            # self.ui.figure_R_CB.setCurrentIndex(1)
            if not self.cc.data_1.loaded:
                self.data_L = self.cc.data_2

        # radio buttons
        # self.ui.serie_1_RB.clicked.connect(self.serie_1_RB_callback)
        # self.ui.serie_2_RB.clicked.connect(self.serie_2_RB_callback)
        # if self.ui.serie_1_RB.isChecked():
        #     self.cc.active_serie = 1
        # else:
        #     self.cc.active_serie = 2
        self.cc.active_serie = 1

        self.ui.action_Load_serie_1.triggered.connect(lambda: self.action_Load_serie_callback(1))
        self.ui.action_Load_serie_2.triggered.connect(lambda: self.action_Load_serie_callback(2))

        self.ui.actionCircle.triggered.connect(self.actionCircle_callback)

        # self.n_slices = self.data.shape[0]
        # self.n_slices = self.cc.data_1.n_slices

        # seting up the callback for the test button --------------------------------------
        # self.ui.test_BTN.clicked.connect(self.test_callback)
        #----------------------------------------------------------------------------------

        # seting up the range of the scrollbar to cope with the number of slices
        # if self.cc.active_serie == 1:
        #     self.ui.slice_C_SB.setMaximum(self.cc.data_1.n_slices - 1)
        #     self.ui.slice_L_SB.setMaximum(self.cc.data_1.n_slices - 1)
        # else:
        #     self.ui.slice_C_SB.setMaximum(self.cc.data_2.n_slices - 1)
        #     self.ui.slice_L_SB.setMaximum(self.cc.data_2.n_slices - 1)
        #
        # if self.cc.data_2.loaded:
        #     self.ui.slice_R_SB.setMaximum(self.cc.data_2.n_slices - 1)

        # combo boxes - for figure views
        # self.ui.figure_L_CB.currentIndexChanged.connect(self.figure_L_CB_callback)
        # self.ui.figure_R_CB.currentIndexChanged.connect(self.figure_R_CB_callback)

        # self.view_L = data_view_widget.SliceBox(self.data_L.shape[1:], self.voxel_size)
        self.view_L = data_view_widget.SliceBox(self.data_L.data_aview.shape[:-1], self.voxel_size)
        self.view_L.setCW(self.win_l, 'c')
        self.view_L.setCW(self.win_w, 'w')
        # self.view_L.setSlice(self.data_L.data[0,:,:])
        self.view_L.setSlice(self.data_L.data_aview[...,0])

        self.view_R = data_view_widget.SliceBox(self.data_R.data_aview.shape[:-1], self.voxel_size)
        self.view_R.setCW(self.win_l, 'c')
        self.view_R.setCW(self.win_w, 'w')
        # self.view_R.setSlice(self.data_R.data[0,:,:])
        self.view_R.setSlice(self.data_R.data_aview[...,0])
        if not self.show_view_L:
            self.view_L.setVisible(False)
        if not self.show_view_R:
            self.view_R.setVisible(False)
        # self.view_L = data_view_widget.SliceBox(self.data_L.shape[1:])
        # self.update_view_L()
        # self.view_R = data_view_widget.SliceBox(self.data_R.shape[1:])
        # self.update_view_R()

        data_viewer_layout = QtGui.QHBoxLayout()
        # data_viewer_layout.addWidget(self.form_widget)
        data_viewer_layout.addWidget(self.view_L)
        data_viewer_layout.addWidget(self.view_R)
        self.ui.viewer_F.setLayout(data_viewer_layout)

        # adding widget for displaying data histograms
        self.hist_widget = hist_widget_old.Hist_widget(self, self.cc)
        hist_viewer_layout = QtGui.QHBoxLayout()
        hist_viewer_layout.addWidget(self.hist_widget)
        # self.ui.histogram_F.setLayout(hist_viewer_layout)

        # connecting callbacks ----------------------------------
        # self.ui.view_L_BTN.clicked.connect(self.view_L_callback)
        # self.ui.view_R_BTN.clicked.connect(self.view_R_callback)
        #
        # # show image data
        # self.ui.show_im_L_BTN.clicked.connect(self.show_im_L_callback)
        # self.ui.show_im_R_BTN.clicked.connect(self.show_im_R_callback)
        #
        # # show label data
        # self.ui.show_labels_L_BTN.clicked.connect(self.show_labels_L_callback)
        # self.ui.show_labels_R_BTN.clicked.connect(self.show_labels_R_callback)
        #
        # # show contours data
        # self.ui.show_contours_L_BTN.clicked.connect(self.show_contours_L_callback)
        # self.ui.show_contours_R_BTN.clicked.connect(self.show_contours_R_callback)

        # main buttons
        # self.ui.calculate_models_BTN.clicked.connect(self.calculate_models_callback)
        # self.ui.run_BTN.clicked.connect(self.run_callback)

        # connecting spin boxes
        # self.ui.hypo_mean_SB.valueChanged.connect(self.hypo_mean_SB_callback)
        # self.ui.hypo_std_SB.valueChanged.connect(self.hypo_std_SB_callback)
        # self.ui.hyper_mean_SB.valueChanged.connect(self.hyper_mean_SB_callback)
        # self.ui.hyper_std_SB.valueChanged.connect(self.hyper_std_SB_callback)
        # self.ui.heal_mean_SB.valueChanged.connect(self.heal_mean_SB_callback)
        # self.ui.heal_std_SB.valueChanged.connect(self.heal_std_SB_callback)

        # connecting scrollbars
        # self.ui.slice_C_SB.valueChanged.connect(self.slider_C_changed)
        # self.ui.slice_L_SB.valueChanged.connect(self.slider_L_changed)
        # self.ui.slice_R_SB.valueChanged.connect(self.slider_R_changed)

        # connecting sliders with their line edit
        self.connect_SL_and_LE()


        self.actionCircle_callback()

        # remove btn
        # self.ui.remove_obj_BTN.clicked.connect(self.remove_obj_BTN_callback)

    def keyPressEvent(self, QKeyEvent):
        print 'key event: ',
        if QKeyEvent.key() == QtCore.Qt.Key_Escape:
            print 'Escape'
            self.view_L.circle_active = False
            self.view_L.area_hist_widget.close()
            self.view_L.setMouseTracking(False)
            self.view_L.updateSlice()

    def test_callback(self):
        self.two_views = not self.two_views
        if self.two_views:
            # self.view_L.setFixedSize(self.view_L.size() / 2)
            self.view_L.resize(self.view_L.pixmap().size() / 2)
        else:
            # self.view_L.setFixedSize(self.view_L.size() * 2)
            self.view_L.resize(self.view_L.pixmap().size() * 2)

    # def update_view_L(self):
    #
    #     self.view_L.setSlice(self.data_L.data[0,:,:])
    #     if not self.show_view_L:
    #         self.view_L.setVisible(False)

    # def update_view_R(self):
    #     # self.view_R = data_view_widget.SliceBox(self.data_R.shape[1:])
    #     self.view_R.setCW(self.win_l, 'c')
    #     self.view_R.setCW(self.win_w, 'w')
    #     self.view_R.setSlice(self.data_R.data[0,:,:])
    #     if not self.show_view_R:
    #         self.view_R.setVisible(False)

    def actionCircle_callback(self):
        self.view_L.circle_active = True
        self.view_L.setMouseTracking(True)
        self.view_L.area_hist_widget = ahw.AreaHistWidget()
        self.view_L.area_hist_widget.show()
        # self.view_R.circle_active = True

    def serie_1_RB_callback(self):
        self.cc.active_serie = 1
        self.cc.actual_data = self.cc.data_1

    def serie_2_RB_callback(self):
        self.cc.active_serie = 2
        self.cc.actual_data = self.cc.data_2

    def connect_SL_and_LE(self):
        pass
        # # win width
        # self.ui.win_wdth_SL.valueChanged.connect(self.win_wdth_SL_changed)
        # ww_val = QtGui.QIntValidator(self.ui.win_wdth_SL.minimum(), self.ui.win_wdth_SL.maximum())
        # self.ui.win_width_LE.setValidator(ww_val)
        # self.ui.win_width_LE.textChanged.connect(self.win_width_LE_changed)
        #
        # # win level
        # self.ui.win_lvl_SL.valueChanged.connect(self.win_lvl_SL_changed)
        # wl_val = QtGui.QIntValidator(self.ui.win_lvl_SL.minimum(), self.ui.win_lvl_SL.maximum())
        # self.ui.win_level_LE.setValidator(wl_val)
        # self.ui.win_level_LE.textChanged.connect(self.win_level_LE_changed)
        #
        # # voxel size
        # self.ui.voxel_size_SB.valueChanged.connect(self.voxel_size_SB_changed)
        # vs_val = QtGui.QIntValidator(self.ui.voxel_size_SB.minimum(), self.ui.voxel_size_SB.maximum())
        # self.ui.voxel_size_LE.setValidator(vs_val)
        # self.ui.voxel_size_LE.textChanged.connect(self.voxel_size_LE_changed)
        #
        # # smoothing - sigma
        # self.ui.sigma_SL.valueChanged.connect(self.sigma_SL_changed)
        # ss_val = QtGui.QIntValidator(self.ui.sigma_SL.minimum(), self.ui.sigma_SL.maximum())
        # self.ui.gaussian_sigma_LE.setValidator(ss_val)
        # self.ui.gaussian_sigma_LE.textChanged.connect(self.gaussian_sigma_LE_changed)
        #
        # # smoothing - sigma_range
        # self.ui.sigma_range_SL.valueChanged.connect(self.sigma_range_SL_changed)
        # sr_val = QtGui.QIntValidator(self.ui.sigma_range_SL.minimum(), self.ui.sigma_range_SL.maximum())
        # self.ui.bilateral_range_LE.setValidator(sr_val)
        # self.ui.bilateral_range_LE.textChanged.connect(self.bilateral_range_LE_changed)
        #
        # # smoothing - sigma_spatial
        # self.ui.sigma_spatial_SL.valueChanged.connect(self.sigma_spatial_SL_changed)
        # sigs_val = QtGui.QIntValidator(self.ui.sigma_spatial_SL.minimum(), self.ui.sigma_spatial_SL.maximum())
        # self.ui.bilateral_spatial_LE.setValidator(sigs_val)
        # self.ui.bilateral_spatial_LE.textChanged.connect(self.bilateral_spatial_LE_changed)
        #
        # # smoothing - tv_weight
        # self.ui.tv_weight_SL.valueChanged.connect(self.tv_weight_SL_changed)
        # tvw_val = QtGui.QIntValidator(self.ui.tv_weight_SL.minimum(), self.ui.tv_weight_SL.maximum())
        # self.ui.tv_weight_LE.setValidator(tvw_val)
        # self.ui.tv_weight_LE.textChanged.connect(self.tv_weight_LE_changed)
        #
        # # alpha
        # self.ui.alpha_SL.valueChanged.connect(self.alpha_SL_changed)
        # alpha_val = QtGui.QIntValidator(self.ui.alpha_SL.minimum(), self.ui.alpha_SL.maximum())
        # self.ui.alpha_LE.setValidator(alpha_val)
        # self.ui.alpha_LE.textChanged.connect(self.alpha_LE_changed)
        #
        # # beta
        # self.ui.beta_SL.valueChanged.connect(self.beta_SL_changed)
        # beta_val = QtGui.QIntValidator(self.ui.beta_SL.minimum(), self.ui.beta_SL.maximum())
        # self.ui.beta_LE.setValidator(beta_val)
        # self.ui.beta_LE.textChanged.connect(self.beta_LE_changed)
        #
        # # frac
        # self.ui.frac_SL.valueChanged.connect(self.frac_SL_changed)
        # frac_val = QtGui.QIntValidator(self.ui.frac_SL.minimum(), self.ui.frac_SL.maximum())
        # self.ui.perc_LE.setValidator(frac_val)
        # self.ui.perc_LE.textChanged.connect(self.perc_LE_changed)
        #
        # # heal_std_k
        # self.ui.heal_std_k_SL.valueChanged.connect(self.heal_std_k_SL_changed)
        # stdh_val = QtGui.QIntValidator(self.ui.heal_std_k_SL.minimum(), self.ui.heal_std_k_SL.maximum())
        # self.ui.k_std_h_LE.setValidator(stdh_val)
        # self.ui.k_std_h_LE.textChanged.connect(self.k_std_h_LE_changed)
        #
        # # tum_std_k
        # self.ui.tum_std_k_SL.valueChanged.connect(self.tum_std_k_SL_changed)
        # stdt_val = QtGui.QIntValidator(self.ui.tum_std_k_SL.minimum(), self.ui.tum_std_k_SL.maximum())
        # self.ui.k_std_t_LE.setValidator(stdt_val)
        # self.ui.k_std_t_LE.textChanged.connect(self.k_std_t_LE_changed)
        #
        # # min_area
        # self.ui.min_area_SL.valueChanged.connect(self.min_area_SL_changed)
        # minarea_val = QtGui.QIntValidator(self.ui.min_area_SL.minimum(), self.ui.min_area_SL.maximum())
        # self.ui.min_area_LE.setValidator(minarea_val)
        # self.ui.min_area_LE.textChanged.connect(self.min_area_LE_changed)
        #
        # # max_area
        # self.ui.max_area_SL.valueChanged.connect(self.max_area_SL_changed)
        # maxarea_val = QtGui.QIntValidator(self.ui.max_area_SL.maximum(), self.ui.max_area_SL.maximum())
        # self.ui.max_area_LE.setValidator(maxarea_val)
        # self.ui.max_area_LE.textChanged.connect(self.max_area_LE_changed)
        #
        # # min_comp
        # self.ui.min_comp_SL.valueChanged.connect(self.min_comp_SL_changed)
        # mincomp_val = QtGui.QIntValidator(self.ui.min_comp_SL.maximum(), self.ui.min_comp_SL.maximum())
        # self.ui.min_comp_LE.setValidator(mincomp_val)
        # self.ui.min_comp_LE.textChanged.connect(self.min_comp_LE_changed)
        #
        # # comp_fact
        # self.ui.comp_fact_SB.valueChanged.connect(self.comp_fact_SB_changed)
        # compf_val = QtGui.QIntValidator(self.ui.comp_fact_SB.maximum(), self.ui.comp_fact_SB.maximum())
        # self.ui.comp_fact_LE.setValidator(compf_val)
        # self.ui.comp_fact_LE.textChanged.connect(self.comp_fact_LE_changed)

    def selection_changed(self, selected, deselected):
        #TODO: povolit oznaceni pouze jednoho objektu?
        if selected.indexes():
            self.selected_objects_labels = [self.table_model.objects[x.row()].label for x in self.ui.objects_TV.selectionModel().selectedRows()]
            # print 'show only', self.selected_objects_labels
            slice = [int(x.center[0]) for x in self.cc.actual_data.lesions if x.label == self.selected_objects_labels[0]][0]
            self.ui.slice_C_SB.setValue(slice)
        else:
            self.selected_objects_labels = [x.label for x in self.table_model.objects]
            # print 'show all', self.selected_objects_labels
        self.cc.objects_filtration(self.selected_objects_labels, min_area=self.ui.min_area_SL.value(), max_area=self.ui.max_area_SL.value())
        #TODO: nasleduje prasarna
        if self.view_L.show_mode == self.view_L.SHOW_LABELS:
            self.show_labels_L_callback()
        if self.view_R.show_mode == self.view_R.SHOW_LABELS:
            self.show_labels_R_callback()

        # self.slider_C_changed(slice)

    def comp_fact_SB_changed(self, value):
        self.ui.comp_fact_LE.setText(str(value))
        self.params['comp_fact'] = value

    def comp_fact_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.comp_fact_SB.setValue(int(value))
            # self.params['comp_fact'] = int(value)
        except:
            pass

    def min_comp_SL_changed(self, value):
        self.ui.min_comp_LE.setText(str(value))
        self.params['min_compactness'] = value
        self.cc.objects_filtration(self.selected_objects_labels)
        # self.fill_table(self.cc.labels[self.cc.filtered_idxs], self.cc.areas[self.cc.filtered_idxs], self.cc.comps[self.cc.filtered_idxs])
        self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.labels, self.cc.filtered_idxs)

    def min_comp_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.min_comp_SL.setValue(int(value))
            # self.params['min_compactness'] = int(value)
            # self.cc.objects_filtration()
            ## self.fill_table(self.cc.labels[self.cc.filtered_idxs], self.cc.areas[self.cc.filtered_idxs], self.cc.comps[self.cc.filtered_idxs])
            #self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.labels, self.cc.filtered_idxs)
        except:
            pass

    def max_area_SL_changed(self, value):
        self.ui.max_area_LE.setText(str(value))
        self.params['max_area'] = value
        self.cc.objects_filtration(self.selected_objects_labels, min_area=self.ui.min_area_SL.value(), max_area=value)
        # self.fill_table(self.cc.labels[self.cc.filtered_idxs], self.cc.areas[self.cc.filtered_idxs], self.cc.comps[self.cc.filtered_idxs])
        self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.labels, self.cc.filtered_idxs)
        # TODO: nasleduje prasarna
        if self.view_L.show_mode == self.view_L.SHOW_LABELS:
            self.show_labels_L_callback()
        if self.view_R.show_mode == self.view_R.SHOW_LABELS:
            self.show_labels_R_callback()

    def max_area_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.max_area_SL.setValue(int(value))
            # self.params['max_area'] = int(value)
            # self.cc.objects_filtration()
            ## self.fill_table(self.cc.labels[self.cc.filtered_idxs], self.cc.areas[self.cc.filtered_idxs], self.cc.comps[self.cc.filtered_idxs])
            # self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.labels, self.cc.filtered_idxs)
        except:
            pass

    def min_area_SL_changed(self, value):
        self.ui.min_area_LE.setText(str(value))
        self.params['min_area'] = value
        self.cc.objects_filtration(self.selected_objects_labels, min_area=value, max_area=self.ui.max_area_SL.value())
        # self.fill_table(self.cc.labels[self.cc.filtered_idxs], self.cc.areas[self.cc.filtered_idxs], self.cc.comps[self.cc.filtered_idxs])
        self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.labels, self.cc.filtered_idxs)
        # TODO: nasleduje prasarna
        if self.view_L.show_mode == self.view_L.SHOW_LABELS:
            self.show_labels_L_callback()
        if self.view_R.show_mode == self.view_R.SHOW_LABELS:
            self.show_labels_R_callback()

    def min_area_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.min_area_SL.setValue(int(value))
            # self.params['min_area'] = int(value)
            # self.cc.objects_filtration()
            ## self.fill_table(self.cc.labels[self.cc.filtered_idxs], self.cc.areas[self.cc.filtered_idxs], self.cc.comps[self.cc.filtered_idxs])
            # self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.labels, self.cc.filtered_idxs)
        except:
            pass

    def tum_std_k_SL_changed(self, value):
        self.ui.k_std_t_LE.setText(str(value))
        self.params['k_std_t'] = value

    def k_std_t_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.tum_std_k_SL.setValue(int(value))
            self.params['k_std_t'] = int(value)
        except:
            pass

    def heal_std_k_SL_changed(self, value):
        self.ui.k_std_h_LE.setText(str(value))
        self.params['k_std_h'] = value

    def k_std_h_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.heal_std_k_SL.setValue(int(value))
            self.params['k_std_h'] = int(value)
        except:
            pass

    def frac_SL_changed(self, value):
        self.ui.perc_LE.setText(str(value))
        self.params['perc'] = value

    def perc_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.frac_SL.setValue(int(value))
            self.params['perc'] = int(value)
        except:
            pass

    def beta_SL_changed(self, value):
        self.ui.beta_LE.setText(str(value))
        self.params['beta'] = value

    def beta_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.beta_SL.setValue(int(value))
            self.params['beta'] = int(value)
        except:
            pass

    def alpha_SL_changed(self, value):
        self.ui.alpha_LE.setText(str(value))
        self.params['alpha'] = value

    def alpha_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.alpha_SL.setValue(int(value))
            self.params['alpha'] = int(value)
        except:
            pass

    def tv_weight_SL_changed(self, value):
        self.ui.tv_weight_LE.setText(str(float(value)/1000))
        self.params['tv_weight'] = value

    def tv_weight_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            w = int(float(value)*1000)
            self.ui.tv_weight_SL.setValue(w)
            self.params['tv_weight'] = w
        except:
            pass

    def sigma_spatial_SL_changed(self, value):
        self.ui.bilateral_spatial_LE.setText(str(value))
        self.params['sigma_spatial'] = value

    def bilateral_spatial_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.sigma_spatial_SL.setValue(int(value))
            self.params['sigma_spatial'] = int(value)
        except:
            pass

    def sigma_range_SL_changed(self, value):
        self.ui.bilateral_range_LE.setText(str(value))
        self.params['sigma_range'] = value

    def bilateral_range_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.sigma_range_SL.setValue(int(value))
            self.params['sigma_range'] = int(value)
        except:
            pass

    def sigma_SL_changed(self, value):
        self.ui.gaussian_sigma_LE.setText(str(value))
        self.params['sigma'] = value

    def gaussian_sigma_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.sigma_SL.setValue(int(value))
            self.params['sigma'] = int(value)
        except:
            pass

    def voxel_size_SB_changed(self, value):
        self.ui.voxel_size_LE.setText(str(value))
        self.params['working_voxel_size'] = value

    def voxel_size_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.voxel_size_SB.setValue(int(value))
            self.params['working_voxel_size'] = int(value)
        except:
            pass

    def win_wdth_SL_changed(self, value):
        self.ui.win_width_LE.setText(str(value))
        self.params['win_width'] = value

    def win_width_LE_changed(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.win_wdth_SL.setValue(int(value))
            self.params['win_width'] = int(value)
        except:
            pass

    def win_lvl_SL_changed(self, value):
        self.ui.win_level_LE.setText(str(value))
        self.params['win_level'] = value

    def win_level_LE_changed(self, value):
        try:  # must be due to the possibility that the character '-' or nothing could be entered
            self.ui.win_lvl_SL.setValue(int(value))
            self.params['win_level'] = int(value)
        except:
            pass

    def load_parameters(self, config_path='config.xml'):
        config = ConfigParser.ConfigParser()
        config.read('config.ini')

        params = dict()

        # an automatic way
        for section in config.sections():
            for option in config.options(section):
                try:
                    params[option] = config.getint(section, option)
                except ValueError:
                    try:
                        params[option] = config.getfloat(section, option)
                    except ValueError:
                        if option == 'voxel_size':
                            str = config.get(section, option)
                            params[option] = np.array(map(int, str.split(', ')))
                        else:
                            params[option] = config.get(section, option)

        return params

    def fill_parameters(self):
        pass
        # # general parameters
        # self.ui.win_wdth_SL.setValue(self.params['win_width'])
        # self.ui.win_width_LE.setText(str(self.params['win_width']))
        #
        # self.ui.win_lvl_SL.setValue(self.params['win_level'])
        # self.ui.win_level_LE.setText(str(self.params['win_level']))
        #
        # self.ui.voxel_size_SB.setValue(self.params['working_voxel_size_mm'])
        # self.ui.voxel_size_LE.setText(str(self.params['working_voxel_size_mm']))
        #
        # # smoothing parameters
        # self.ui.sigma_SL.setValue(self.params['sigma'])
        # self.ui.gaussian_sigma_LE.setText(str(self.params['sigma']))
        #
        # self.ui.sigma_range_SL.setValue(self.params['sigma_range'])
        # self.ui.bilateral_range_LE.setText(str(self.params['sigma_range']))
        #
        # self.ui.sigma_spatial_SL.setValue(self.params['sigma_spatial'])
        # self.ui.bilateral_spatial_LE.setText(str(self.params['sigma_spatial']))
        #
        # self.ui.tv_weight_SL.setValue(self.params['tv_weight'])
        # self.ui.tv_weight_LE.setText(str(self.params['tv_weight']))
        #
        # # color model parameters
        # self.ui.frac_SL.setValue(self.params['perc'])
        # self.ui.perc_LE.setText(str(self.params['perc']))
        #
        # self.ui.heal_std_k_SL.setValue(self.params['k_std_h'])
        # self.ui.k_std_h_LE.setText(str(self.params['k_std_h']))
        #
        # self.ui.tum_std_k_SL.setValue(self.params['k_std_t'])
        # self.ui.k_std_t_LE.setText(str(self.params['k_std_t']))
        #
        # # localization parameters
        # self.ui.alpha_SL.setValue(self.params['alpha'])
        # self.ui.alpha_LE.setText(str(self.params['alpha']))
        #
        # self.ui.beta_SL.setValue(self.params['beta'])
        # self.ui.beta_LE.setText(str(self.params['beta']))
        #
        # # self.ui.min_area_SL.setValue(self.params['min_area'])
        # # self.ui.min_area_LE.setText(str(self.params['min_area']))
        # self.ui.min_area_SL.setValue(0)
        # self.ui.min_area_LE.setText('0')
        #
        # # self.ui.max_area_SL.setValue(self.params['max_area'])
        # # self.ui.max_area_LE.setText(str(self.params['max_area']))
        #
        # self.ui.max_area_SL.setValue(0)
        # self.ui.max_area_LE.setText('0')
        #
        # self.ui.min_comp_SL.setValue(self.params['min_compactness'])
        # self.ui.min_comp_LE.setText(str(self.params['min_compactness']))
        #
        # self.ui.comp_fact_SB.setValue(self.params['comp_fact'])
        # self.ui.comp_fact_LE.setText(str(self.params['comp_fact']))

    def hypo_mean_SB_callback(self, value):
        self.statusBar().showMessage('Hypodense model updated thru spin box.')
        rv = scista.norm(value, self.cc.models['rv_hypo'].std())
        self.cc.models['rv_hypo'] = rv
        self.hist_widget.update_hypo_rv(rv)
        self.hist_widget.update_figures()

    def hypo_std_SB_callback(self, value):
        self.statusBar().showMessage('Hypodense model updated thru spin box.')
        rv = scista.norm(self.cc.models['rv_hypo'].mean(), value)
        self.cc.models['rv_hypo'] = rv
        self.hist_widget.update_hypo_rv(rv)
        self.hist_widget.update_figures()

    def hyper_mean_SB_callback(self, value):
        self.statusBar().showMessage('Hyperdense model updated thru spin box.')
        rv = scista.norm(value, self.cc.models['rv_hyper'].std())
        self.cc.models['rv_hyper'] = rv
        self.hist_widget.update_hyper_rv(rv)
        self.hist_widget.update_figures()

    def hyper_std_SB_callback(self, value):
        self.statusBar().showMessage('Hyperdense model updated thru spin box.')
        rv = scista.norm(self.cc.models['rv_hyper'].mean(), value)
        self.cc.models['rv_hyper'] = rv
        self.hist_widget.update_hyper_rv(rv)
        self.hist_widget.update_figures()

    def heal_mean_SB_callback(self, value):
        self.statusBar().showMessage('Healthy model updated thru spin box.')
        rv = scista.norm(value, self.cc.models['rv_heal'].std())
        self.cc.models['rv_heal'] = rv
        self.hist_widget.update_heal_rv(rv)
        self.hist_widget.update_figures()

    def heal_std_SB_callback(self, value):
        self.statusBar().showMessage('Healthy model updated thru spin box.')
        rv = scista.norm(self.cc.models['rv_heal'].mean(), value)
        self.cc.models['rv_heal'] = rv
        self.hist_widget.update_heal_rv(rv)
        self.hist_widget.update_figures()

    def scroll_event(self, value, who):
        if who == 0:  # left viewer
            new = self.actual_slice_L + value
            if (new < 0) or (new >= self.data_L.n_slices):
                return
        elif who == 1:  # right viewer
            new = self.actual_slice_R + value
            if (new < 0) or (new >= self.data_R.n_slices):
                return

    def slider_C_changed(self, val):
        if val == self.actual_slice_L:
            return

        if (val >= 0) and (val < self.data_L.n_slices):
            diff = val - self.actual_slice_L
            self.actual_slice_L = val
        else:
            return

        new_slice_R = self.actual_slice_R + diff
        if new_slice_R < 0:
            new_slice_R = 0
        elif new_slice_R >= self.data_R.n_slices:
            new_slice_R = self.data_R.n_slices - 1

        self.actual_slice_R = new_slice_R

        self.ui.slice_L_SB.setValue(self.actual_slice_L)
        self.ui.slice_R_SB.setValue(self.actual_slice_R)

        im_L = self.get_image('L')
        im_R = self.get_image('R')
        if self.show_mode_L == SHOW_CONTOURS:
            labels_L = self.data_L.labels_filt[self.actual_slice_L, :, :]
        else:
            labels_L = None
        if self.show_mode_R == SHOW_CONTOURS:
            labels_R = self.data_R.labels_filt[self.actual_slice_R, :, :]
        else:
            labels_R = None

        self.view_L.setSlice(im_L, contours=labels_L)
        self.view_R.setSlice(im_R, contours=labels_R)

        self.ui.slice_number_L_LBL.setText('%i/%i' % (self.actual_slice_L + 1, self.data_L.n_slices))
        self.ui.slice_number_C_LBL.setText('slice # = %i/%i' % (self.actual_slice_L + 1, self.data_L.n_slices))

    def slider_L_changed(self, val):
        if val == self.actual_slice_L:
            return

        if (val >= 0) and (val < self.data_L.n_slices):
            self.actual_slice_L = val
        else:
            return

        self.ui.slice_C_SB.setValue(self.actual_slice_L)

        im_L = self.get_image('L')
        if self.show_mode_L == SHOW_CONTOURS:
            labels_L = self.data_L.labels_filt[self.actual_slice_L, :, :]
        else:
            labels_L = None

        self.view_L.setSlice(im_L, contours=labels_L)

        self.ui.slice_number_L_LBL.setText('%i/%i' % (self.actual_slice_L + 1, self.data_L.n_slices))
        self.ui.slice_number_C_LBL.setText('slice # = %i/%i' % (self.actual_slice_L + 1, self.data_L.n_slices))

    def slider_R_changed(self, val):
        if (val >= 0) and (val < self.data_R.n_slices):
            self.actual_slice_R = val
        else:
            return

        self.ui.slice_number_R_LBL.setText('%i/%i' % (self.actual_slice_R + 1, self.data_R.n_slices))

        im_R = self.get_image('R')
        if self.show_mode_R == SHOW_CONTOURS:
            labels_R = self.data_R.labels_filt[self.actual_slice_R, :, :]
        else:
            labels_R = None

        self.view_R.setSlice(im_R, contours=labels_R)

    def calculate_models_callback(self):
        self.statusBar().showMessage('Calculating intensity models...')
        if self.params['zoom']:
            data = self.data_zoom(self.cc.actual_data.data, self.cc.actual_data.voxel_size, self.params['working_voxel_size_mm'])
            mask = self.data_zoom(self.cc.actual_data.mask, self.cc.actual_data.voxel_size, self.params['working_voxel_size_mm'])
        else:
            data = tools.resize3D(self.cc.actual_data.data, self.params['scale'], sliceId=0)
            mask = tools.resize3D(self.cc.actual_data.mask, self.params['scale'], sliceId=0)
        self.cc.models = self.cc.calculate_intensity_models(data, mask)
        self.statusBar().showMessage('Intensity models calculated.')
        self.update_models()

    def update_models(self):
        self.hist_widget.update_heal_rv(self.cc.models['rv_heal'])
        self.hist_widget.update_hypo_rv(self.cc.models['rv_hypo'])
        self.hist_widget.update_hyper_rv(self.cc.models['rv_hyper'])

        self.ui.heal_mean_SB.setValue(self.cc.models['rv_heal'].mean())
        self.ui.heal_std_SB.setValue(self.cc.models['rv_heal'].std())

        self.ui.hypo_mean_SB.setValue(self.cc.models['rv_hypo'].mean())
        self.ui.hypo_std_SB.setValue(self.cc.models['rv_hypo'].std())

        self.ui.hyper_mean_SB.setValue(self.cc.models['rv_hyper'].mean())
        self.ui.hyper_std_SB.setValue(self.cc.models['rv_hyper'].std())

        self.hist_widget.update_figures()

    def run_callback(self):
        # Viewer_3D.run(self.data)
        # viewer = Viewer_3D.Viewer_3D(self.data)
        # viewer.show()

        # run localization
        self.statusBar().showMessage('Localization started...')
        self.cc.run()
        self.update_models()

        # self.form_widget.update_figures()
        # self.hist_widget.update_figures()
        self.labels = self.cc.actual_data.labels
        # self.labels = self.cc.labels

        # filling table with objects
        # self.fill_table(self.cc.labels[self.cc.filtered_idxs], self.cc.areas[self.cc.filtered_idxs], self.cc.comps[self.cc.filtered_idxs])
        self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.objects, self.cc.filtered_idxs)
        self.selected_objects_labels = [x.label for x in self.table_model.objects]
        # seting up range of area slider
        areas = [x.area for x in self.cc.actual_data.lesions]
        self.ui.min_area_SL.setMaximum(max(areas))
        self.ui.min_area_SL.setMinimum(min(areas))
        self.ui.max_area_SL.setMaximum(max(areas))
        self.ui.max_area_SL.setMinimum(min(areas))
        self.ui.max_area_SL.setValue(max(areas))

        self.ui.show_labels_L_BTN.setEnabled(True)
        self.ui.show_contours_L_BTN.setEnabled(True)

        # self.cc.areas = np.array([10, 20, 30, 8])
        # self.cc.comps = np.array([51, 60, 70, 80])
        # self.cc.objects = np.array([[0,0,-1,3,3],[0,-1,-1,3,3],[-1,-1,-1,-1,-1],[1,1,-1,-1,2],[1,-1,-1,2,2]])
        # self.cc.labels = np.unique(self.cc.objects)[1:]
        # self.cc.n_objects = len(np.unique(self.cc.objects)) - 1
        # self.fill_table(self.cc.labels, self.cc.areas, self.cc.comps)

    def fill_table(self, lesions, labels, idxs):
        lesions_filtered = [x for x in lesions if x.label in idxs]
        # labels_filtered = np.where(labels in idxs, labels, 0)
        labels_filtered = labels * np.in1d(labels, idxs).reshape(labels.shape)
        self.table_model = mtm.MyTableModel(lesions_filtered, labels_filtered)
        self.ui.objects_TV.setModel(self.table_model)

        # tableview selection changed
        self.ui.objects_TV.selectionModel().selectionChanged.connect(self.selection_changed)

        # self.ui.objects_TV.selectionModel().selectionChanged.connect(self.selection_changed)

    # def fill_table(self, labels, areas, comps):
    #     features = np.vstack((labels, areas, comps)).T
    #     self.table_model = mtm.MyTableModel(features)
    #     self.ui.objects_TV.setModel(self.table_model)
    #     self.ui.objects_TV.selectionModel().selectionChanged.connect(self.selection_changed)
    #     # self.tableview.verticalHeader().setVisible(True)


    def view_L_callback(self):
        self.show_view_L = not self.show_view_L
        self.view_L.setVisible(self.show_view_L)

        # enabling and disabling other toolbar icons
        self.ui.show_im_L_BTN.setEnabled(self.show_view_L)

        if self.show_view_L and self.data_L.labels is not None:
            self.ui.show_labels_L_BTN.setEnabled(True)
            self.ui.show_contours_L_BTN.setEnabled(True)
        else:
            self.ui.show_labels_L_BTN.setEnabled(False)
            self.ui.show_contours_L_BTN.setEnabled(False)


        self.statusBar().showMessage('Left view set to %s' % self.show_view_L)
        # print 'view_1 set to', self.show_view_1

        # self.form_widget.update_figures()
        self.view_L.update()

    def view_R_callback(self):
        self.show_view_R = not self.show_view_R
        self.view_R.setVisible(self.show_view_R)

        # enabling and disabling other toolbar icons
        self.ui.show_im_R_BTN.setEnabled(not self.ui.show_im_R_BTN.isEnabled())

        if self.show_view_R and self.data_R.labels is not None:
            self.ui.show_labels_R_BTN.setEnabled(True)
            self.ui.show_contours_R_BTN.setEnabled(True)
        else:
            self.ui.show_labels_R_BTN.setEnabled(False)
            self.ui.show_contours_R_BTN.setEnabled(False)

        self.statusBar().showMessage('Right view set to %s' % self.show_view_R)
        # print 'view_2 set to', self.show_view_2

        # self.form_widget.update_figures()
        self.view_R.update()

    def show_im_L_callback(self):
        self.show_mode_L = SHOW_IM
        self.view_L.show_mode = self.view_L.SHOW_IM

        im = self.get_image('L')
        self.view_L.setSlice(im)

        self.statusBar().showMessage('data_L set to im')

    def show_im_R_callback(self):
        self.show_mode_R = SHOW_IM
        self.view_R.show_mode = self.view_R.SHOW_IM

        im = self.get_image('R')
        self.view_R.setSlice(im)

        self.statusBar().showMessage('data_R set to im')

    def show_labels_L_callback(self):
        self.show_mode_L = SHOW_LABELS
        self.view_L.show_mode = self.view_L.SHOW_LABELS

        im = self.get_image('L')
        self.view_L.setSlice(im)

        self.statusBar().showMessage('data_L set to labels')

    def show_labels_R_callback(self):
        self.show_mode_R = SHOW_LABELS
        self.view_R.show_mode = self.view_R.SHOW_LABELS

        im = self.get_image('R')
        self.view_R.setSlice(im)

        self.statusBar().showMessage('data_R set to labels')

    def show_contours_L_callback(self):
        self.show_mode_L = SHOW_CONTOURS
        self.view_L.show_mode = self.view_L.SHOW_CONTOURS

        im = self.get_image('L')
        labels = self.data_L.labels_filt[self.actual_slice_L, :, :]
        self.view_L.setSlice(im, contours=labels)

        self.statusBar().showMessage('data_L set to contours')

    def show_contours_R_callback(self):
        self.show_mode_R = SHOW_CONTOURS
        self.view_R.show_mode = self.view_R.SHOW_CONTOURS

        im = self.get_image('R')
        labels = self.data_R.labels_filt[self.actual_slice_R, :, :]
        self.view_R.setSlice(im, contours=labels)

        self.statusBar().showMessage('data_R set to contours')

    def figure_L_CB_callback(self):
        if self.ui.figure_L_CB.currentIndex() == 0:
            self.data_L = self.cc.data_1
        elif self.ui.figure_L_CB.currentIndex() == 1:
            self.data_L = self.cc.data_2

        if self.actual_slice_L >= self.data_L.n_slices:
            self.actual_slice_L = self.data_L.n_slices - 1

        self.ui.slice_L_SB.setMaximum(self.data_L.n_slices - 1)
        self.ui.slice_C_SB.setMaximum(self.data_L.n_slices - 1)

        if (self.data_L.labels is not None) and self.show_view_L:
            self.ui.show_labels_L_BTN.setEnabled(True)
            self.ui.show_contours_L_BTN.setEnabled(True)
        else:
            self.ui.show_labels_L_BTN.setEnabled(False)
            self.ui.show_contours_L_BTN.setEnabled(False)

        self.view_L.reinit(self.data_L.shape[1:])
        self.show_im_L_callback()

    def figure_R_CB_callback(self):
        if self.ui.figure_R_CB.currentIndex() == 0:
            self.data_R = self.cc.data_1
        elif self.ui.figure_R_CB.currentIndex() == 1:
            self.data_R = self.cc.data_2

        if self.actual_slice_R >= self.data_R.n_slices:
            self.actual_slice_R = self.data_R.n_slices - 1

        self.ui.slice_R_SB.setMaximum(self.data_R.n_slices - 1)

        if (self.data_R.labels is not None) and self.show_view_R:
            self.ui.show_labels_R_BTN.setEnabled(True)
            self.ui.show_contours_R_BTN.setEnabled(True)
        else:
            self.ui.show_labels_R_BTN.setEnabled(False)
            self.ui.show_contours_R_BTN.setEnabled(False)

        self.view_R.reinit(self.data_R.shape[1:])
        self.show_im_R_callback()

    def action_Load_serie_callback(self, serie_number):
        fname = QtGui.QFileDialog.getOpenFileName(self, 'Open file', self.params['data_dir'])
        print 'Does not work yet.'
        print fname

    def remove_obj_BTN_callback(self):
        indexes = self.ui.objects_TV.selectionModel().selectedRows()
        # for i in reversed(indexes):
        #     self.table_model.removeRow(i.row())
        for i in indexes:
            obj = self.table_model.objects[i.row()]
            self.remove_object(obj)

        # self.cc.actual_data.filtered_idxs = [x.label for x in self.cc.actual_data.lesions]
        self.selected_objects_labels = [x.label for x in self.cc.actual_data.lesions]
        self.cc.objects_filtration(self.selected_objects_labels, min_area=self.ui.min_area_SL.value(), max_area=self.ui.max_area_SL.value())
        self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.objects, self.cc.filtered_idxs)
        #TODO: nasleduje prasarna
        if self.view_L.show_mode == self.view_L.SHOW_LABELS:
            self.show_labels_L_callback()
        if self.view_R.show_mode == self.view_R.SHOW_LABELS:
            self.show_labels_R_callback()

    def remove_object(self, obj):
        lbl = obj.label
        im = self.cc.actual_data.objects == lbl
        self.cc.actual_data.objects[np.nonzero(im)] = self.params['bgd_label']
        self.cc.actual_data.labels[np.nonzero(im)] = self.params['healthy_label']

        self.cc.actual_data.lesions.remove(obj)
        print 'removed label', lbl

    def get_image(self, site):
        im = None
        if site == 'L':
            if self.show_mode_L == SHOW_IM or self.show_mode_L == SHOW_CONTOURS:
                # im = self.data_L.data[self.actual_slice_L, :, :]
                im = self.data_L.data_aview[...,self.actual_slice_L]
            elif self.show_mode_L == SHOW_LABELS:
                # im = self.data_L.labels[self.actual_slice_L, :, :]
                im = self.data_L.labels_aview[...,self.actual_slice_L]
        elif site == 'R':
            if self.show_mode_R == SHOW_IM or self.show_mode_R == SHOW_CONTOURS:
                # im = self.data_R.data[self.actual_slice_R, :, :]
                im = self.data_R.data_aview[...,self.actual_slice_R]
            elif self.show_mode_R == SHOW_LABELS:
                # im = self.data_R.labels[self.actual_slice_R, :, :]
                im = self.data_R.labels_aview[...,self.actual_slice_R]
        return im

    def run(self, im, labels, healthy_label, hypo_label, hyper_label, slice_axis=2, disp_smoothed=False):
        if slice_axis == 0:
            im = np.transpose(im, (1, 2, 0))
            labels = np.transpose(labels, (1, 2, 0))
        app = QtGui.QApplication(sys.argv)
        le = Lession_editor(im, labels, healthy_label, hypo_label, hyper_label, disp_smoothed)
        le.show()
        sys.exit(app.exec_())


################################################################################
################################################################################
if __name__ == '__main__':
    fname = ''
    fnames = list()

    # 2 hypo, 1 on the border --------------------
    # arterial 0.6mm - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_183_46324212_arterial_0.6_B30f-.pklz'
    # venous 0.6mm - good
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_183_46324212_venous_0.6_B20f-.pklz'
    # venous 5mm - ok, but wrong approach
    # TODO: study ID 29 - 3/3, 2 velke spoji v jeden
    # slice = 17
    fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_183_46324212_venous_5.0_B30f-.pklz')
    fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_183_46324212_arterial_5.0_B30f-.pklz')

    # hypo in venous -----------------------
    # arterial - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_186_49290986_venous_0.6_B20f-.pklz'
    # venous - good
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_186_49290986_arterial_0.6_B20f-.pklz'
    # fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_186_49290986_venous_0.6_B20f-.pklz')
    # fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_186_49290986_arterial_0.6_B20f-.pklz')

    # hyper, 1 on the border -------------------
    # arterial 0.6mm - not that bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_239_61293268_DE_Art_Abd_0.75_I26f_M_0.5-.pklz'
    # venous 5mm - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_239_61293268_DE_Ven_Abd_0.75_I26f_M_0.5-.pklz'

    # shluk -----------------
    # arterial 5mm
    # fname = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_180_49509315_arterial_5.0_B30f-.pklz'
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_180_49509315_arterial_0.6_B20f-.pklz'
    # TODO: study ID 18 - velke najde (hyper v arterialni fazi), 1/2 z malych
    # fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_180_49509315_venous_5.0_B30f-.pklz')
    # fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_180_49509315_arterial_5.0_B30f-.pklz')

    # targeted
    # arterial 0.6mm - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_238_54280551_Abd_Arterial_0.75_I26f_3-.pklz'
    # venous 0.6mm - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_238_54280551_Abd_Venous_0.75_I26f_3-.pklz'

    # TODO: study ID 25 - 2/2
    # fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_185_48441644_venous_5.0_B30f-.pklz')
    # fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_185_48441644_arterial_5.0_B30f-.pklz')

    # TODO: study ID 21 - 0/2, nenasel ani jeden pekny hypo
    # slice = 6
    # fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_186_49290986_venous_5.0_B30f-.pklz')
    # fnames.append('/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_186_49290986_arterial_5.0_B30f-.pklz')

    # runing application -------------------------
    app = QtGui.QApplication(sys.argv)
    le = Lession_editor(fnames)
    le.show()
    sys.exit(app.exec_())


# NOTES and TODOS ----------
# TODO: vizualizace kontur
# TODO: min/max_area_SL_changed  - je tam prasarna, aby se aktualizovala vizualizace labelu uz pri pohybu slideru
# TODO: kontury 'contours' (obrysy) nefunguji
# TODO: Okno nejde zmensit, jen zvetsit. Asi kvuli velikosti obrazku, kdyz se da visible(False), vse funguje.
# TODO: Menit velikost tak, aby bylo mozne okno horizontalne menit (kvuli zapinani a vypinani pohledu)


import sys
from PyQt4.QtCore import QTimeLine
from PyQt4.QtGui import *

class FaderWidget(QWidget):

    def __init__(self, old_widget, new_widget):
    
        QWidget.__init__(self, new_widget)
        
        self.old_pixmap = QPixmap(new_widget.size())
        old_widget.render(self.old_pixmap)
        self.pixmap_opacity = 1.0
        
        self.timeline = QTimeLine()
        self.timeline.valueChanged.connect(self.animate)
        self.timeline.finished.connect(self.close)
        self.timeline.setDuration(333)
        self.timeline.start()
        
        self.resize(new_widget.size())
        self.show()
    
    def paintEvent(self, event):
    
        painter = QPainter()
        painter.begin(self)
        painter.setOpacity(self.pixmap_opacity)
        painter.drawPixmap(0, 0, self.old_pixmap)
        painter.end()
    
    def animate(self, value):
    
        self.pixmap_opacity = 1.0 - value
        self.repaint()

class StackedWidget(QStackedWidget):

    def __init__(self, parent = None):
        QStackedWidget.__init__(self, parent)
    
    def setCurrentIndex(self, index):
        self.fader_widget = FaderWidget(self.currentWidget(), self.widget(index))
        QStackedWidget.setCurrentIndex(self, index)
    
    def setPage1(self):
        self.setCurrentIndex(0)
    
    def setPage2(self):
        self.setCurrentIndex(1)


if __name__ == "__main__":

    app = QApplication(sys.argv)
    
    window = QWidget()
    
    stack = StackedWidget()
    stack.addWidget(QCalendarWidget())
    editor = QTextEdit()
    editor.setPlainText("Hello world! "*100)
    stack.addWidget(editor)
    
    page1Button = QPushButton("Page 1")
    page2Button = QPushButton("Page 2")
    page1Button.clicked.connect(stack.setPage1)
    page2Button.clicked.connect(stack.setPage2)
    
    layout = QGridLayout(window)
    layout.addWidget(stack, 0, 0, 1, 2)
    layout.addWidget(page1Button, 1, 0)
    layout.addWidget(page2Button, 1, 1)
    
    window.show()
    
    sys.exit(app.exec_())